"""
Built-in constraints

All classes need a transform class. Note, unlike sklearn, transform can copy
or overwrite input depending on copy attribute.
"""

from abc import (ABC as _ABC, abstractmethod as _abstractmethod)

import numpy as _np

class Constraint(_ABC):
    """ Abstract class for constraints """

    @_abstractmethod
    def transform(self, A):
        """ Transform A input based on constraint """

class ConstraintNonneg(Constraint):
    """
    Non-negativity constraint. All negative entries made 0.

    Parameters
    ----------
    copy : bool
        Make copy of input data, A; otherwise, overwrite (if mutable)
    """
    def __init__(self, copy=False):
        """ A must be non-negative"""
        self.copy = copy

    def transform(self, A):
        """ Apply nonnegative constraint"""
        if self.copy:
            return A*(A > 0)
        else:
            A *= (A > 0)
            return A

class ConstraintNorm(Constraint):
    """
    Normalization constraint.

    Parameters
    ----------
    axis : int
        Which axis of input matrix A to apply normalization acorss.
    copy : bool
        Make copy of input data, A; otherwise, overwrite (if mutable)
    """
    def __init__(self, axis=-1, copy=False):
        """Normalize along axis"""
        self.copy = copy
        if not ((axis == 0) | (axis == 1) | (axis == -1)):
            raise ValueError('Axis must be 0,1, or -1')
        self.axis = axis

    def transform(self, A):
        """ Apply normalization constraint """
        if self.copy:
            if self.axis == 0:
                return A / A.sum(axis=self.axis)[None, :]
            else:
                return A / A.sum(axis=self.axis)[:, None]
        else:
            if A.dtype != _np.float:
                raise TypeError('A.dtype must be float for in-place math (copy=False)')

            if self.axis == 0:
                A /= A.sum(axis=self.axis)[None, :]
            else:
                A /= A.sum(axis=self.axis)[:, None]
            return A



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Author: Stéphan Pissot
A minimalistic launcher to start the tools
"""
import sys
from PyQt5.Qt import qApp
from PyQt5.QtWidgets import QApplication, QWidget, QPushButton, QLabel
from octavvs import preprocessing, clustering, mcr_als

def start_preprocessing():
    preprocessing.MyMainWindow.run_octavvs_application(isChild=True)
def start_clustering():
    clustering.MyMainWindow.run_octavvs_application(isChild=True)
def start_mcrals():
    mcr_als.MyMainWindow.run_octavvs_application(isChild=True)

def main():
   app = QApplication(sys.argv)
   widget = QWidget()

   label=QLabel(widget)
   label.setText('OCTAVVS')
   label.move(10,10)

   button1 = QPushButton(widget)
   button1.setText("Preprocessing")
   button1.move(10,42)
   button1.clicked.connect(start_preprocessing)

   button2 = QPushButton(widget)
   button2.setText("Clustering")
   button2.move(10,72)
   button2.clicked.connect(start_clustering)

   button3 = QPushButton(widget)
   button3.setText("MCR-ALS")
   button3.move(10,102)
   button3.clicked.connect(start_mcrals)

   widget.setWindowTitle("OCTAVVS Launcher")
   widget.show()
   qApp.lastWindowClosed.connect(qApp.quit);
   sys.exit(app.exec_())

#if __name__ == '__main__':
#    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  4 16:38:00 2020

@author: carl
"""

from .atm_correction import atmospheric
from .util import load_reference, nonnegative
from .mie_correction import rmiesc
from .ptir import normalize_mirage



# -*- coding: utf-8 -*-
"""
@author: syahr
"""
import gc
#import sys
import csv
import glob
import os
from sklearn.cluster import MiniBatchKMeans
#import traceback
from os.path import basename, dirname
from datetime import datetime
from pkg_resources import resource_filename
import argparse
from scipy.optimize import nnls as nnls

from PyQt5.QtWidgets import QFileDialog, QMessageBox,  QDialog
from PyQt5.Qt import QMainWindow,qApp
from PyQt5.QtCore import QThread, pyqtSignal, pyqtSlot #QCoreApplication, QObject, QRunnable, QThreadPool
from PyQt5 import uic

import matplotlib
matplotlib.use('QT5Agg')
import matplotlib.pyplot as plt
import numpy as np
from sklearn.metrics import mean_squared_error as msea
import scipy as sc
from skimage.draw import polygon
import pickle

# from .pymcr_new.regressors import OLS, NNLS
# from .pymcr_new.constraints import ConstraintNonneg, ConstraintNorm
from .mcr import ftir_function as ff
from octavvs.algorithms import correction as mc
from octavvs.ui import (FileLoader, ImageVisualizer, OctavvsMainWindow, NoRepeatStyle, uitools)

Ui_MainWindow = uic.loadUiType(resource_filename(__name__, "mcr/mcr_final_loc.ui"))[0]
Ui_MainWindow2 = uic.loadUiType(resource_filename(__name__, "mcr/mcr_roi_sub.ui"))[0]
Ui_DialogAbout = uic.loadUiType(resource_filename(__name__, "mcr/about.ui"),
                                from_imports=True, import_from='octavvs')[0]

class DialogAbout(QDialog, Ui_DialogAbout):
    def __init__(self, parent=None):
        QDialog.__init__(self, parent)
        qApp.installEventFilter(self)
        self.setupUi(self)

class Second(QMainWindow, Ui_MainWindow2):
    def __init__(self, parent=None):
        super(Second, self).__init__(parent=None)
        qApp.installEventFilter(self)
        self.setupUi(self)
        self.pushButtonClose.clicked.connect(self.close)
        self.pushButtonReset.clicked.connect(self.Plot_ROI.resetAll)
        self.pushButtonRemove.clicked.connect(self.Plot_ROI.removeOne)
        self.comboBox_roi.currentIndexChanged.connect(self.ImageProjection)
        self.mainwin = parent

    @pyqtSlot()
    def ImageProjection(self):
        img = self.mainwin.plot_whitelight.img
        self.comboBox_roi.setEnabled(img is not None)
        if img is None:
            self.comboBox_roi.setCurrentIndex(0)

        self.Plot_ROI.setSize(self.mainwin.projection.shape)
        if self.comboBox_roi.currentIndex() == 0:
            self.Plot_ROI.setImage(self.mainwin.projection, self.mainwin.comboBoxCmaps.currentText())
        else:
            self.Plot_ROI.setImage(img)

    @pyqtSlot(str)
    def setCmap(self, cmap):
        self.Plot_ROI.setCmap(cmap)

    @pyqtSlot()
    def resetAll(self):
        self.Plot_ROI.resetAll()
        self.hide()


class MyMainWindow(OctavvsMainWindow, Ui_MainWindow):

    projectionUpdated = pyqtSignal()
    loadedFile = pyqtSignal()

    @classmethod
    def program_name(cls):
        "Return the name of the program that this main window represents"
        return 'MCR-ALS'

    def __init__(self, parent=None):
        super().__init__(parent)
        self.default_dir =  os.path.dirname(__file__) + "\\mcr\\"+'dat.initi'
        self.lineEditSuffix.setText('_purest')
        self.pushButtonLocal.setEnabled(False)
        self.pushButtonLoad.clicked.connect(self.Load)
        self.lock_all(False)
        self.progressBar.hide()
        self.pushButtonStop.hide()
        self.comboBoxMethod.currentIndexChanged.connect(self.ImageProjection)
        self.horizontalSliderWavenumber.valueChanged.connect(self.Wavenumbercal)
        self.comboBoxCmaps.currentTextChanged.connect(self.plot_visual.setCmap)

        self.comboBoxCmaps.currentTextChanged.connect(self.ImageProjection)
        self.loadedFile.connect(self.plot_visual.clearMarkings)

        self.pushButtonSVD.clicked.connect(self.SVDprocess)
        self.comboBoxInitial.currentIndexChanged.connect(self.InitialCondition)
        self.pushButtonPurestCal.clicked.connect(self.run)
        self.checkBoxSaveInit.toggled.connect(self.SaveInit)
        self.checkBoxSavePurest.toggled.connect(self.SavePurest)
        self.pushButtonExpandSpectra.clicked.connect(self.ExpandSpec)
        self.pushButtonExpandProjection.clicked.connect(self.ExpandProj)
        self.pushButtonExpandSVD.clicked.connect(self.ExpandSVD)
        self.pushButtonExpandInitSpect.clicked.connect(self.ExpandInitSpect)
        self.pushButtonExpandPurConc.clicked.connect(self.ExpandPurConc)
        self.pushButtonExpandPurSp.clicked.connect(self.ExpandPurSp)
        self.pushButtonWhitelight.clicked.connect(self.WhiteRead)
        self.pushButtonStop.clicked.connect(self.killer)
        self.lineEditHeight.editingFinished.connect(self.ValidationX)
        self.lineEditWidth.editingFinished.connect(self.ValidationY)

        self.pushButtonLocal.clicked.connect(self.roi)

        self.projection = None
        self.roiDialog = Second(self)
        self.roiDialog.Plot_ROI.captured.connect(self.setRoi)
        self.roiDialog.Plot_ROI.captured.connect(self.plot_visual.setRoi)
        self.projectionUpdated.connect(self.roiDialog.ImageProjection)
        self.projectionUpdated.connect(self.ExpandProjU)

        self.comboBoxCmaps.currentTextChanged.connect(self.roiDialog.setCmap)
        self.loadedFile.connect(self.roiDialog.resetAll)

        self.checkBoxInvert.toggled.connect(self.Invert)
        self.About = DialogAbout()

        self.actionAbout.triggered.connect(self.About.show)
        self.roiDialog.actionAbout.triggered.connect(self.About.show)
        self.comboBoxImp.currentIndexChanged.connect(self.VisSpectra)
        self.spinBoxWlength.valueChanged.connect(self.VisSpectra)
        self.spinBoxPoly.valueChanged.connect(self.VisSpectra)

        

        self.spinBoxWlength.hide()
        self.spinBoxPoly.hide()
        self.labelJ.hide()
        self.labelWl.hide()
        self.labelPl.hide()



        self.post_setup()




    def closeEvent(self, event):
        msgBox = QMessageBox()
        msgBox.setIcon(QMessageBox.Question)
        msgBox.setText("Warning")
        msgBox.setInformativeText('Are you sure to close the window ?')
        msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        msgBox.setDefaultButton(QMessageBox.No)
        reply = msgBox.exec_()
        if reply == QMessageBox.Yes:
            plt.close('all')
            self.roiDialog.close()
            self.About.close()
            if hasattr(self, 'calpures'):
                self.calpures.stop()
#            self.killer_renew()
#            qApp.quit()
        else:
            event.ignore()


    def roi(self):
        self.comboBoxInitial.setCurrentIndex(0)
        self.roiDialog.show()

    def Load(self):
        try:
            infile = open(self.default_dir,'rb')
            dire = pickle.load(infile)
            infile.close()
        except:
            dire = os.path.dirname(__file__) 
        
        if self.comboBoxSingMult.currentIndex() == 1:
            self.pushButtonLocal.setEnabled(False)
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            name,_ = QFileDialog.getOpenFileName(self,"Open Matrix File", dire,"Matrix File (*.mat)", options=options)
            self.foldername = dirname(name)
            if self.foldername:
                outfile = open(self.default_dir,'wb')
                pickle.dump(self.foldername,outfile)
                outfile.close()
                self.progressBar.show()
                self.search_whole_folder(self.foldername)
            else:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Critical)
                msg.setText('No file is loaded')
                msg.setInformativeText("Please select a file")
                msg.setWindowTitle("Warning")
                msg.setStandardButtons(QMessageBox.Ok )
                msg.exec_()

        elif self.comboBoxSingMult.currentIndex() == 0:
            self.pushButtonLocal.setEnabled(True)
            self.progressBar.hide()
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            filename, _ = QFileDialog.getOpenFileName(self,"Open Matrix File", dire,"Matrix File (*.mat)", options=options)
            if filename:
                outfile = open(self.default_dir,'wb')
                pickle.dump(dirname(filename),outfile)
                outfile.close()
                
                self.allnames = [filename]
                self.coord = []
                self.clear_prev()
                self.lineEditTotal.setText(str(1))
                self.initialization(filename)
                self.lineEditFileNumber.setText(str(1))

        elif self.comboBoxSingMult.currentIndex() == 2:
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            fileNames, _ = QFileDialog.getOpenFileNames(self,"Open Matrix File", dire,"Matrix File (*.mat)", options=options)
            if fileNames:
                outfile = open(self.default_dir,'wb')
                pickle.dump(dirname(fileNames[0]),outfile)
                outfile.close()

                count = 0
                name =  {}
                foldername = dirname(fileNames[0])
                for file in fileNames:
                    name[count] = file
                    count += 1
                # count = 0

                w = csv.writer(open(foldername+"//Fileall.csv", "w"))
                for key, val in sorted(name.items(), key=lambda item: item[1]):
#                    for key, val in sorted(name.items()):
                    w.writerow([key, val])

                self.allnames = name
                self.nfiles = count
                self.lineEditTotal.setText(str(count))
                self.lineEditFileNumber.setText(str(1))
                self.initialization(name[0])
                self.SVDprocess()


    def initialization(self,fileName):

        self.lineEditFilename.setText(basename(fileName))
        self.labelDirectory.setText(dirname(fileName))

        try:
            self.lock_all(True)
            self.sx, self.sy, self.pin,self.wavenumber, self.spo = ff.readmat(fileName)
            self.spo = mc.nonnegative(self.spo)


            self.lineEditLength.setText(str(len(self.wavenumber)))
            self.labelMinwn.setText(str("%.2f" % np.min(self.wavenumber)))
            self.labelMaxwn.setText(str("%.2f" % np.max(self.wavenumber)))
            self.lineEditWavenumber.setText(str("%.2f" % np.min(self.wavenumber)))

            kmeans = MiniBatchKMeans(n_clusters=8, random_state=0).fit(self.spo.T)
            self.raw = kmeans.cluster_centers_
            self.raw = self.raw.T


            self.VisSpectra()
            self.Select_spectra()


        except:
            # This exception handling must be made much more specific. What exception, what lines?
            raise
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setText('The .mat file is not FTIR File')
            msg.setInformativeText("Please select another file !")
            msg.setWindowTitle("Warning")
            msg.setStandardButtons(QMessageBox.Ok )
            msg.exec_()

        self.plot_whitelight.load(fileName.replace(fileName.split('.0')[-1],'.jpg'))
#        self.plot_whitelight.load(os.path.splitext(fileName)[0]+'.jpg')
        self.loadedFile.emit()



    def Select_spectra(self):
        win = int(self.spinBoxWlength.value())
        pol = int(self.spinBoxPoly.value())
        if self.comboBoxImp.currentIndex() == 0:
            self.sp = mc.nonnegative(self.spo)
            self.p = self.sp.reshape(np.shape(self.pin),order='C')
        elif self.comboBoxImp.currentIndex() == 1:
            self.sp = sc.signal.savgol_filter(self.spo.T,win, polyorder = pol,deriv=1)
            self.sp = self.sp.T
            self.sp = mc.nonnegative(self.sp)
            self.p = self.sp.reshape(np.shape(self.pin),order='C')

        elif self.comboBoxImp.currentIndex() == 2:
            self.sp = sc.signal.savgol_filter(self.spo.T,win, polyorder = pol,deriv=2)
            self.sp = self.sp.T
            self.sp = mc.nonnegative(self.sp)
            self.p = self.sp.reshape(np.shape(self.pin),order='C')

        try:
            x = int(self.lineEditHeight.text())
            y = int(self.lineEditWidth.text())
            z = int(self.lineEditLength.text())
            self.p = np.reshape(self.p,(z,x,y))
        except ValueError:
            self.lineEditWidth.setText(str(self.sx))
            self.lineEditHeight.setText(str(self.sy))

        self.ImageProjection()




    def VisSpectra(self):
        self.Select_spectra()
        win = int(self.spinBoxWlength.value())
        pol = int(self.spinBoxPoly.value())

        if self.comboBoxImp.currentIndex() == 0:
            self.yvis = self.raw.copy()
            self.spinBoxWlength.hide()
            self.spinBoxPoly.hide()
            self.labelJ.hide()
            self.labelWl.hide()
            self.labelPl.hide()


        elif self.comboBoxImp.currentIndex() == 1:
            self.yvis= sc.signal.savgol_filter(self.raw.T,win, polyorder = pol,deriv=1)
            self.yvis= self.yvis.T
            self.spinBoxWlength.show()
            self.spinBoxPoly.show()
            self.labelJ.show()
            self.labelWl.show()
            self.labelPl.show()


        else:
            self.yvis= sc.signal.savgol_filter(self.raw.T,win, polyorder = pol,deriv=2)
            self.yvis= self.yvis.T

            self.spinBoxWlength.show()
            self.spinBoxPoly.show()
            self.labelJ.show()
            self.labelWl.show()
            self.labelPl.show()


        self.yvis = mc.nonnegative(self.yvis)
        self.plot_specta.canvas.ax.clear()
        self.plot_specta.canvas.ax.plot(self.wavenumber,self.yvis)
        if self.checkBoxInvert.isChecked():
            self.plot_specta.Invert()
        self.plot_specta.canvas.fig.tight_layout()
        self.plot_specta.canvas.draw()

        self.ExpandSpecU()

    def Invert(self):
        self.ExpandSpecU()
        self.ExpandPurSpU()
        # self.ExpandInitSpectU()
        
        self.plot_specta.Invert()
        self.plot_specta.canvas.fig.tight_layout()
        self.plot_specta.canvas.draw()

        self.plotPurestSpectra.Invert()
        self.plotPurestSpectra.canvas.fig.tight_layout()
        self.plotPurestSpectra.canvas.draw()

        if self.comboBoxMethod.currentIndex() == 2:
            self.Wavenumbercal()

        if hasattr(self,'nr') :
            if self.comboBoxInitial.currentIndex() == 0:
                self.InitialVis()



    def ValidationX(self):
        x = int(self.lineEditHeight.text())
        y = int(self.lineEditWidth.text())
        z = int(self.lineEditLength.text())
        xy = int(self.sx*self.sy)

        if x == 0 or y == 0:
            self.lineEditHeight.setText(str(self.sy))
            self.lineEditWidth.setText(str(self.sx))
            x = self.sx
            y = self.sy

        elif int(x*y) != xy:
            excess = np.mod(xy,x)
            if excess == 0 :
                y=xy/x
                y = int(y)
                self.lineEditWidth.setText(str(y))
            else:
                self.lineEditHeight.setText(str(self.sy))
                self.lineEditWidth.setText(str(self.sx))
                x = self.sx
                y = self.sy
        else:
            self.lineEditHeight.setText(str(x))
            self.lineEditWidth.setText(str(y))

        self.p = np.reshape(self.p,(z,x,y))
        self.ImageProjection()

    def ValidationY(self):
        x = int(self.lineEditHeight.text())
        y = int(self.lineEditWidth.text())
        z = int(self.lineEditLength.text())
        xy = int(self.sx*self.sy)

        if x == 0 or y == 0:
            self.lineEditHeight.setText(str(self.sy))
            self.lineEditWidth.setText(str(self.sx))
            x = self.sx
            y = self.sy

        elif int(x*y) != xy:
            excess = np.mod(xy,y)
            if excess == 0:
                x=xy/y
                x = int(x)
                self.lineEditHeight.setText(str(x))
            else:
                self.lineEditHeight.setText(str(self.sy))
                self.lineEditWidth.setText(str(self.sx))
                x = self.sx
                y = self.sy
        else:
            self.lineEditHeight.setText(str(x))
            self.lineEditWidth.setText(str(y))

        self.p = np.reshape(self.p,(z,x,y))
        self.ImageProjection()




    def search_whole_folder(self, foldername):
        count = 0
        name =  {}
        a = [x[0] for x in os.walk(foldername)]
        for i in a:
            os.chdir(i)
            for file in glob.glob('*.mat'):
                name[count] = str(i+'/'+file)
                count += 1


        if count != 0:
            w = csv.writer(open(foldername+"//Fileall.csv", "w"))
            for key, val in sorted(name.items(), key=lambda item: item[1]):
#            for key, val in sorted(name.items()):
                w.writerow([key, val])

            self.allnames = name
            self.nfiles = count
            self.lineEditTotal.setText(str(count))
            self.lineEditFileNumber.setText(str(1))
            self.initialization(name[0])
            self.SVDprocess()
        else:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setText('The .mat file does not exist in this directory')
            msg.setInformativeText("Please select another directory !")
            msg.setWindowTitle("Warning")
            msg.setStandardButtons(QMessageBox.Ok )
            msg.exec_()

    def WhiteRead(self):
       options = QFileDialog.Options()
       options |= QFileDialog.DontUseNativeDialog
       white, _ = QFileDialog.getOpenFileName(self,"Open White Light Image", "","images(*.jpg *.png)", options=options)
       if white:
           self.plot_whitelight.load(white)
           self.projectionUpdated.emit()

    def ImageProjection(self):

        if  self.comboBoxMethod.currentIndex() == 0:
            self.horizontalSliderWavenumber.setEnabled(False)
            self.projection = ff.proarea(self.p,self.wavenumber)
        if  self.comboBoxMethod.currentIndex() == 1:
            self.lineEditWavenumber.setEnabled(False)
            self.horizontalSliderWavenumber.setEnabled(False)
            self.projection = ff.promip(self.p)
        if  self.comboBoxMethod.currentIndex() == 2:
            self.lineEditWavenumber.setEnabled(True)
            self.horizontalSliderWavenumber.setEnabled(True)
            self.wavenumv = float(self.lineEditWavenumber.text())
            self.projection = ff.prowavenum(self.p,self.wavenumber,self.wavenumv)

        self.plot_visual.setImage(self.projection, self.comboBoxCmaps.currentText())

#        self.plot_visual.canvas.ax.clear()
#        self.plot_visual.canvas.ax.imshow(self.projection,str(self.comboBoxCmaps.currentText()))
#        self.plot_visual.canvas.fig.tight_layout()
#        self.plot_visual.canvas.draw()

        self.projectionUpdated.emit()


    def ExpandProj(self):
        nr = self.spinBoxSVDComp.value()
        fig = plt.figure("Image Projection",tight_layout={'pad':.5})
        fig.clear()
        ax = fig.gca()
        ax.imshow(self.projection,str(self.comboBoxCmaps.currentText()))
        plt.colorbar(ax.imshow(self.projection,str(self.comboBoxCmaps.currentText())))
        if len(self.splot ) != 1 :
            for j in range(0,nr):
                ax.plot(self.pos[j,0],self.pos[j,1],marker='p', color = 'black')
        fig.show()


    @pyqtSlot()
    def ExpandProjU(self):
        nr = self.spinBoxSVDComp.value()
        if plt.fignum_exists("Image Projection"):
            fig = plt.figure("Image Projection",tight_layout={'pad':.5})
            fig.clear()
            ax = fig.gca()
            ax.imshow(self.projection,str(self.comboBoxCmaps.currentText()))
            plt.colorbar(ax.imshow(self.projection,str(self.comboBoxCmaps.currentText())))
            if len(self.splot ) != 1 :
                for j in range(0,nr):
                    ax.plot(self.pos[j,0],self.pos[j,1],marker='p', color = 'black')
            fig.canvas.draw_idle()
        else:
            pass



    def ExpandSVD(self):
        if len(self.splot ) != 1 :
            # plt.close("SVD Plot")
            fig = plt.figure("SVD Plot",tight_layout={'pad':.5})
            fig.clear()
            ax = fig.gca() 
            ax.plot(self.xplot,self.splot,'-o')
            fig.show()
        else:
            pass


    def ExpandSVDU(self):
        if plt.fignum_exists("SVD Plot"):
            fig = plt.figure("SVD Plot",tight_layout={'pad':.5})
            ax = fig.gca()
            ax.clear()
            ax.plot(self.xplot,self.splot,'-o')
            fig.canvas.draw_idle()
        else:
            pass


    def ExpandInitSpect(self):
        # plt.close("Initial")
        fig = plt.figure("Initial",tight_layout={'pad':.5})
        ax = fig.gca() 
        if len(self.insp) != 1:
            if self.comboBoxInitial.currentIndex() == 0:
                ax.plot(self.wavenumber,self.insp.T)
                if self.checkBoxInvert.isChecked():
                    ax.invert_xaxis()
            else:
                ax.plot(np.arange(self.sx*self.sy),self.incon.T)
            fig.show()


    def ExpandInitSpectU(self,x,y):
        if plt.fignum_exists("Initial"):
            fig = plt.figure("Initial",tight_layout={'pad':.5})
            ax = fig.gca()
            ax.clear()
            ax.plot(x,y)
            if self.checkBoxInvert.isChecked():
                ax.invert_xaxis()
            fig.canvas.draw_idle()
        else:
            pass



    def Wavenumbercal(self):
        nnow1 = ((np.max(self.wavenumber) - np.min(self.wavenumber))
        *float(self.horizontalSliderWavenumber.value())/10000.0 + np.min(self.wavenumber))
        nnow1 = "%.2f" % nnow1
        self.lineEditWavenumber.setText(str(nnow1))
        self.wavenumv = float(self.lineEditWavenumber.text())
        self.projection = ff.prowavenum(self.p,self.wavenumber,self.wavenumv)

        self.plot_specta.canvas.ax.clear()
        self.plot_specta.canvas.ax.plot(self.wavenumber,self.yvis)
        self.plot_specta.canvas.ax.axvline(x=self.wavenumv)
        if self.checkBoxInvert.isChecked():
            self.plot_specta.Invert()
        self.plot_specta.canvas.fig.tight_layout()
        self.plot_specta.canvas.draw()

        self.plot_visual.setImage(self.projection, self.comboBoxCmaps.currentText())
        self.projectionUpdated.emit()
        self.ExpandSpecU()


    def ExpandSpec(self):
        # plt.close("Spectra")
        fig = plt.figure("Spectra",tight_layout={'pad':.5})
        ax = fig.gca()
        ax.clear()
        ax.plot(self.wavenumber,self.yvis)
        if self.comboBoxMethod.currentIndex() == 2:
                ax.axvline(x=self.wavenumv)
        if self.checkBoxInvert.isChecked():
            ax.invert_xaxis()

        ax.set_xlabel("Wavenumber(1/cm)",fontsize=14)
        ax.set_ylabel("Absorption(arb. units)",fontsize=14)
        ax.tick_params(axis='both',direction='in', length=8, width=1)
        ax.tick_params(axis='both',which='major',labelsize=14)
        fig.show()


    def ExpandSpecU(self):
        if plt.fignum_exists("Spectra"):
            fig = plt.figure("Spectra",tight_layout={'pad':.5})
            ax = fig.gca()
            ax.clear()
            ax.plot(self.wavenumber,self.yvis)
            if self.comboBoxMethod.currentIndex() == 2:
                ax.axvline(x=self.wavenumv)
            if self.checkBoxInvert.isChecked():
                ax.invert_xaxis()
            fig.canvas.draw_idle()
        else:
            pass



    def ExpandPurConc(self):
        if len(self.copt)  != 1:
            # plt.close("Purest Concentrations")
            fig = plt.figure("Purest Concentrations",tight_layout={'pad':.5})
            ax = fig.subplots()
            ax.plot(np.arange(len(self.copt)),self.copt)
            fig.show()

    def ExpandPurConcU(self,copt):
        if plt.fignum_exists("Purest Concentrations"):
            fig = plt.figure("Purest Concentrations",tight_layout={'pad':.5})
            ax = fig.gca()
            ax.clear()
            ax.plot(np.arange(len(copt)),copt)
            fig.canvas.draw_idle()
        else:
            pass


    def ExpandPurSp(self):
        if len(self.sopt) != 1:
            # plt.close("Purest Spectra")
            fig = plt.figure("Purest Spectra",tight_layout={'pad':.5})
            ax = fig.subplots()
            if self.checkBoxInvert.isChecked():
                ax.invert_xaxis()
            ax.plot(self.wavenumber,self.sopt)
            fig.show()

    def ExpandPurSpU(self):
        if plt.fignum_exists("Purest Spectra") and len(self.sopt) == len(self.wavenumber):
            fig = plt.figure("Purest Spectra",tight_layout={'pad':.5})
            ax = fig.gca()
            ax.clear()
            ax.plot(self.wavenumber, self.sopt)
            if self.checkBoxInvert.isChecked():
                ax.invert_xaxis()
            fig.canvas.draw_idle()
        else:
            pass

    def SVDprocess(self):
        win = int(self.spinBoxWlength.value())
        pol = int(self.spinBoxPoly.value())

        self.nr = self.spinBoxSVDComp.value()
        if self.nr < 20:
            nplot = self.nr+5
        else:
            nplot = self.nr

        if self.comboBoxImp.currentIndex() == 1:
            self.sp = sc.signal.savgol_filter(self.spo.T,win, polyorder = pol,deriv=1)
            self.sp = self.sp.T

        elif self.comboBoxImp.currentIndex() == 2:
            self.sp= sc.signal.savgol_filter(self.spo.T,win, polyorder = pol,deriv=2)
            self.sp= self.sp.T
        else:
            self.sp = self.spo.copy()




        if not self.coord:
            self.sp = mc.nonnegative(self.sp)
            self.u, self.s, self.v = np.linalg.svd(self.sp)

            self.xplot = np.arange(nplot)
            self.splot =self.s[0:nplot]
            self.SVDPlot()
        else:
            nx, ny = int(self.lineEditWidth.text()),int(self.lineEditHeight.text())

            self.roi = np.zeros((ny, nx))
            vertex_col_coords,vertex_row_coords = np.array(self.coord).T
            fill_row_coords, fill_col_coords = polygon(
                    vertex_row_coords, vertex_col_coords, self.roi.shape)
            self.roi[fill_row_coords, fill_col_coords] = 1
            self.rem = self.roi * self.projection
            img_d = np.reshape(self.rem,(nx*ny,1))
            self.ind =  np.where(img_d > 0)[0]

            sp_new = mc.nonnegative(self.sp[:,self.ind])
            self.u, self.s, self.v = np.linalg.svd(sp_new)

            nplot = min(nplot, len(self.s))
            self.xplot = np.arange(nplot)
            self.splot =self.s[0:nplot]

            self.plotSVD.canvas.ax.clear()
            self.plotSVD.canvas.ax.plot(self.xplot,self.splot,'-o')
            self.plotSVD.canvas.draw()
            self.ExpandSVDU()

            per = float(self.lineEditNoisePercent.text())
            self.f = per*0.01


            if self.comboBoxInitial.currentIndex() == 0:
                self.incon = [0,0]
                self.labelInitial.setText("Initial Spectra*")
                self.insp, points = ff.initi_simplisma(sp_new,self.nr,self.f)
                self.pos = np.zeros((self.nr,2))

                for i in range(0,self.nr):
                    self.pos[i,0] = self.ind[points[i]] % nx
                    self.pos[i,1] = self.ind[points[i]] // nx
                self.plot_visual.addPoints(self.pos)
                self.InitialVis()


    def InitialVis(self):
        self.plotInitSpec.canvas.ax.clear()
        self.plotInitSpec.canvas.ax.plot(self.wavenumber,self.insp.T)
        if self.checkBoxInvert.isChecked():
            self.plotInitSpec.Invert()
        self.plotInitSpec.canvas.fig.tight_layout()
        self.plotInitSpec.canvas.draw()
        self.ExpandInitSpectU(self.wavenumber,self.insp.T)




    def SVDPlot(self):
        self.plotPurestConc.canvas.ax.clear()
        self.plotPurestConc.canvas.draw()
        self.plotPurestSpectra.canvas.ax.clear()
        self.plotPurestSpectra.canvas.draw()

        self.plotSVD.canvas.ax.clear()
        self.plotSVD.canvas.ax.plot(self.xplot,self.splot,'-o')
        self.plotSVD.canvas.draw()
#        if self.comboBoxPurest.currentIndex() == 0:
        self.ExpandSVDU()
        self.InitialCondition()


    def InitialCondition(self):
        nr = self.spinBoxSVDComp.value()
        per = float(self.lineEditNoisePercent.text())
        self.f = per*0.01

        if self.comboBoxInitial.currentIndex() == 0:
            self.incon = [0,0]
            self.labelInitial.setText("Initial Spectra*")
            self.insp, points = ff.initi_simplisma(self.sp,nr,self.f)
            self.pos = np.array([points % self.projection.shape[0], points // self.projection.shape[1]]).T
            self.plot_visual.setImage(self.projection, self.comboBoxCmaps.currentText())
            self.plot_visual.addPoints(self.pos)
            self.InitialVis()
            self.ExpandProjU()
            if self.checkBoxSaveInit.isChecked():
                self.save_data_init(self.insp.T)


        else:
            self.insp = [0,0]
            self.labelInitial.setText("Initial Concentration*")
            self.incon, __ = ff.initi_simplisma(self.sp.T,nr,self.f)
            self.plotInitSpec.canvas.ax.clear()
            self.plot_visual.clearMarkings()
            self.plotInitSpec.canvas.ax.plot(np.arange(self.sx*self.sy),self.incon.T)
            self.plotInitSpec.canvas.fig.tight_layout()
            self.plotInitSpec.canvas.draw()
            self.ExpandInitSpectU(np.arange(self.sx*self.sy),self.incon.T)
            self.ImageProjection()
            if self.checkBoxSaveInit.isChecked():
                self.save_data_init(self.incon.T)


    def lockmcr(self):
        self.comboBoxInitial.setEnabled(True)
        self.pushButtonExpandInitSpect.setEnabled(True)
        self.comboBoxRegressor.setEnabled(True)
        self.lineEditNoisePercent.setEnabled(True)
        self.checkBoxSaveInit.setEnabled(True)
        self.lineEditPurIter.setText('700')
        # self.SVDprocess()


    def run(self):

        self.plotPurestSpectra.canvas.ax.clear()
        self.plotPurestSpectra.canvas.draw()

        self.plotPurestConc.canvas.ax.clear()
        self.plotPurestConc.canvas.draw()


        if self.checkBoxSavePurest.isChecked():
             if self.comboBoxSingMult.currentIndex() == 0:
                 self.runsingle()
             else:
                 self.runall()

        else:
            msgBox = QMessageBox()
            msgBox.setIcon(QMessageBox.Question)
            msgBox.setText("Warning")
            msgBox.setInformativeText('The Purest Spectra Will not be saved')
            msgBox.setStandardButtons(QMessageBox.Yes| QMessageBox.No)
            msgBox.setDefaultButton(QMessageBox.No)
            reply = msgBox.exec_()
            if reply == QMessageBox.Yes:
                if self.comboBoxSingMult.currentIndex() == 0:
                    self.runsingle()
                else:
                    self.runall()
            else:
                pass


    def runall(self):
        nr = self.spinBoxSVDComp.value()
        f = 0.01*float(self.lineEditNoisePercent.text())
        max_iter = int(self.lineEditPurIter.text())
        stopping_error = float(self.lineEditTol.text())
        init = self.comboBoxInitial.currentIndex()
        win = int(self.spinBoxWlength.value())
        pol = int(self.spinBoxPoly.value())
        implement = int(self.comboBoxImp.currentIndex())


        self.progressBar.setEnabled(True)
        self.progressBar.setMaximum(self.nfiles+1)
        self.lineEditStatus.setText('Multiple files')
        self.calpures = Multiple_Calculation(self.allnames, nr, f,max_iter,
                                             stopping_error, init,win, pol,implement)


        self.calpures.purest.connect(self.finished_single)
        self.calpures.DataInit.connect(self.finished_mcr_all)
        self.calpures.start()
        self.pushButtonStop.show()
        self.pushButtonPurestCal.setEnabled(False)

    def finished_mcr_all(self,count, filename):
        self.progressBar.setValue(count)
        self.lineEditFileNumber.setText(str(count))
        self.lineEditFilename.setText(basename(filename))
        self.initialization(filename)
        self.SVDprocess()


    @pyqtSlot(list)
    def setRoi(self, roi):
        self.coord = roi

    def runsingle(self):
        if not self.coord:
            self.nfiles = 1
            self.runall()
        else:
            self.runsingle_roi()

    # def runsingle_noroi(self):
    #     max_iter = int(self.lineEditPurIter.text())
    #     tol_percent = float(self.lineEditTol.text())
    #     f = 0.01*float(self.lineEditNoisePercent.text())
    #     self.SVDprocess()
    #     nr = self.spinBoxSVDComp.value()
    #     init = self.comboBoxInitial.currentIndex()
    #     self.InitialCondition()
    #     self.calpures = single_report(self.sp,nr,f, max_iter,tol_percent,init)

    #     self.calpures = Multiple_Calculation(self.allnames, nr, f,max_iter,
    #                                          stopping_error, init,win, pol,implement)




    #     self.calpures.purest.connect(self.finished_single)
    #     self.calpures.start()
    #     self.pushButtonStop.show()
    #     self.pushButtonPurestCal.setEnabled(False)



    def killer(self):
        msgBox = QMessageBox()
        msgBox.setIcon(QMessageBox.Question)
        msgBox.setText("Warning")
        msgBox.setInformativeText('Are you sure to terminate the calculation ?')
        msgBox.setStandardButtons(QMessageBox.Yes| QMessageBox.No)
        msgBox.setDefaultButton(QMessageBox.No)
        reply = msgBox.exec_()
        if reply == QMessageBox.Yes:
            self.pushButtonStop.hide()
            self.pushButtonPurestCal.setEnabled(True)
            if hasattr(self, 'calpures'):
                self.calpures.stop()
            self.killer_renew()
        else:
            pass

    def runsingle_roi(self):
        self.comboBoxInitial.setCurrentIndex(0)
        nx, ny = int(self.lineEditWidth.text()),int(self.lineEditHeight.text())
        max_iter = int(self.lineEditPurIter.text())
        tol_percent = float(self.lineEditTol.text())

        self.roi = np.zeros((ny, nx))
        vertex_col_coords, vertex_row_coords = np.array(self.coord).T
        fill_row_coords, fill_col_coords = polygon(
                vertex_row_coords, vertex_col_coords, self.roi.shape)
        self.roi[fill_row_coords, fill_col_coords] = 1
        self.rem = self.roi * self.projection

        img_d = np.reshape(self.rem,(int(nx*ny),1))
        self.ind =  np.where(img_d > 0)[0]

        nr = self.spinBoxSVDComp.value()

        sp_new = mc.nonnegative(self.sp[:,self.ind])
        self.u, self.s, self.v = np.linalg.svd(sp_new)
        if nr < 20:
            nplot = nr+5
        else:
            nplot = nr

        self.xplot = np.arange(nplot)
        self.splot =self.s[0:nplot]

        self.plotPurestConc.canvas.ax.clear()
        self.plotPurestConc.canvas.draw()
        self.plotPurestSpectra.canvas.ax.clear()
        self.plotPurestSpectra.canvas.draw()

        self.plotSVD.canvas.ax.clear()
        self.plotSVD.canvas.ax.plot(self.xplot,self.splot,'-o')
        self.plotSVD.canvas.draw()
        self.ExpandSVDU()

        per = float(self.lineEditNoisePercent.text())
        self.f = per*0.01


        if self.comboBoxInitial.currentIndex() == 0:
            self.incon = [0,0]
            self.labelInitial.setText("Initial Spectra*")
            self.insp, points = ff.initi_simplisma(sp_new,nr,self.f)
            self.InitialVis()
            # self.plotInitSpec.canvas.ax.clear()
            # self.plotInitSpec.canvas.ax.plot(self.wavenumber,self.insp.T)
            # if self.checkBoxInvert.isChecked():
            #     self.plotInitSpec.Invert()
            # self.plotInitSpec.canvas.fig.tight_layout()
            # self.plotInitSpec.canvas.draw()
            if self.checkBoxSaveInit.isChecked():
                self.save_data_init(self.insp.T)
#            self.pos = np.array([points % self.projection.shape[0], points // self.projection.shape[1]]).T
            self.pos = np.zeros((nr,2))

            for i in range(0,nr):
                self.pos[i,0] = self.ind[points[i]] % nx
                self.pos[i,1] = self.ind[points[i]] // nx
                self.plot_visual.addPoints(self.pos)


        nrow, ncol = np.shape(sp_new)

        s = sc.linalg.diagsvd(self.s,nrow, ncol)
        u = self.u[:,0:nr]
        s = s[0:nr,0:nr]
        v = self.v[0:nr,:]
        self.dn = u @ s @ v


        nrow, ncol = np.shape(self.dn)
        dauxt = np.zeros((ncol,nrow))
        aux=self.dn.T

        for i in range(0,ncol):
            dauxt[i,:]=aux[i,:]/np.sqrt(np.sum(aux[i,:]*aux[i,:]))


        f = 0.01*float(self.lineEditNoisePercent.text())
        init = self.comboBoxInitial.currentIndex()



        self.calpures = single_report(sp_new, nr, f, max_iter,tol_percent, init)
        self.calpures.purest.connect(self.finished_single_roi)
        self.calpures.start()
        self.pushButtonStop.show()
        self.pushButtonPurestCal.setEnabled(False)

    def finished_single_roi(self, itern,error,status,copt,sopt):
        self.copt = copt
        self.sopt = sopt

        self.ExpandPurConcU(copt)
        self.ExpandPurSpU()



        nr = self.spinBoxSVDComp.value()
        nx, ny = int(self.lineEditWidth.text()),int(self.lineEditHeight.text())
        bea = np.zeros((int(nx*ny),nr))
        for i in range(0,nr):
            bea[self.ind,i] = copt[:,i]


        self.plotPurestSpectra.canvas.ax.clear()
        self.plotPurestSpectra.canvas.ax.plot(self.wavenumber,sopt)
        if self.checkBoxInvert.isChecked():
            self.plotPurestSpectra.Invert()
        self.plotPurestSpectra.canvas.fig.tight_layout()
        self.plotPurestSpectra.canvas.draw()

        self.plotPurestConc.canvas.ax.clear()
        self.plotPurestConc.canvas.ax.plot(np.arange(len(bea)),bea)
        self.plotPurestConc.canvas.fig.tight_layout()
        self.plotPurestConc.canvas.draw()


        self.lineEdit_Niter.setText(str(itern))
        self.lineEdit_Error.setText(str(round(error,5)))
        self.lineEditStatus.setText(status)


        if (status == 'Max iterations reached') or (status == 'converged'):
            self.pushButtonPurestCal.setEnabled(True)
            self.pushButtonStop.hide()
            if self.checkBoxSavePurest.isChecked():
                self.save_data(bea,sopt)



    def save_data(self, copt, sopt):
        win = int(self.spinBoxWlength.value())
        pol = int(self.spinBoxPoly.value())
        met = int(self.comboBoxImp.currentIndex())
        meta = np.zeros((3,len(sopt.T)))
        meta[0,0] = met
        meta[1,0] = win
        meta[2,0] = pol


        # if self.checkBoxSavePurest.isChecked():
        auxi = np.concatenate((meta,sopt,copt), axis = 0)
        namef = self.lineEditFilename.text()
        namef = namef.replace('.mat','')
        np.savetxt(self.folpurest+'/'+namef+self.lineEditSuffix.text()+'.csv', auxi, delimiter=',')



    def SaveInit(self):
        if self.checkBoxSaveInit.isChecked():
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            self.folinit = QFileDialog.getExistingDirectory(self,"Open the input data")
        else:
            pass


    def save_data_init(self, data):
        if self.checkBoxSaveInit.isChecked():
            # auxi = np.concatenate((sopti,copti), axis = 0)
            namef = self.lineEditFilename.text()
            namef = namef.replace('.mat','')
            if self.comboBoxInitial.currentIndex() == 0:
                suf= '_init_spec'
            else:
                suf = '_init_con'
            np.savetxt(self.folinit+'/'+namef+suf+'.csv', data, delimiter=',')


    def SavePurest(self):
        if self.checkBoxSavePurest.isChecked():
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            self.folpurest = QFileDialog.getExistingDirectory(self,"Open the input data")
        else:
            pass

    def finished_single(self, itern,error,status,copt,sopt):
        self.copt = copt
        self.sopt = sopt

        self.ExpandPurConcU(copt)
        self.ExpandPurSpU()



        self.plotPurestSpectra.canvas.ax.clear()
        self.plotPurestSpectra.canvas.ax.plot(self.wavenumber,sopt)
        if self.checkBoxInvert.isChecked():
            self.plotPurestSpectra.Invert()
        self.plotPurestSpectra.canvas.fig.tight_layout()
        self.plotPurestSpectra.canvas.draw()

        self.plotPurestConc.canvas.ax.clear()
        self.plotPurestConc.canvas.ax.plot(np.arange(len(copt)),copt)
        self.plotPurestConc.canvas.fig.tight_layout()
        self.plotPurestConc.canvas.draw()

        self.lineEdit_Niter.setText(str(itern))
        self.lineEdit_Error.setText(str(round(error,5)))
        self.lineEditStatus.setText(status)

        if (status == 'Max iterations reached') or (status == 'converged'):
            if self.checkBoxSavePurest.isChecked():
                self.save_data(copt,sopt)
            if self.comboBoxSingMult.currentIndex() == 1 and (int(self.lineEditFileNumber.text())==
                                                             int(self.lineEditTotal.text())) :
                self.pushButtonPurestCal.setEnabled(True)
                self.progressBar.setValue(self.nfiles+1)
                self.lineEditStatus.setText('DONE')
                self.pushButtonStop.hide()
            elif self.comboBoxSingMult.currentIndex() == 0:
                self.pushButtonPurestCal.setEnabled(True)
                # self.progressBar.setValue(self.nfiles+1)
                self.lineEditStatus.setText('DONE')
                self.pushButtonStop.hide()



    def lock_all(self,Stat):
        self.pushButtonExpandSpectra.setEnabled(Stat)
        self.comboBoxMethod.setEnabled(Stat)
        self.pushButtonExpandProjection.setEnabled(Stat)
        self.pushButtonExpandSVD.setEnabled(Stat)
        self.pushButtonExpandInitSpect.setEnabled(Stat)
        self.pushButtonSVD.setEnabled(Stat)
        self.spinBoxSVDComp.setEnabled(Stat)
        self.lineEditPurIter.setEnabled(Stat)
        self.lineEditTol.setEnabled(Stat)
        self.lineEditNoisePercent.setEnabled(Stat)
        self.lineEditStatus.setEnabled(Stat)
        self.checkBoxSaveInit.setEnabled(Stat)
        self.checkBoxSavePurest.setEnabled(Stat)
        # self.pushButtonPurestCal.setEnabled(Stat)
        self.pushButtonExpandPurSp.setEnabled(Stat)
        self.pushButtonExpandPurConc.setEnabled(Stat)
        self.comboBoxCmaps.setEnabled(Stat)
        self.comboBoxInitial.setEnabled(Stat)
        self.comboBoxRegressor.setEnabled(Stat)
        self.lineEditWavenumber.setEnabled(Stat)
        self.lineEditKeyword.setEnabled(Stat)
        self.lineEditSuffix.setEnabled(Stat)
        self.lineEditFilename.setEnabled(Stat)
        self.lineEditFileNumber.setEnabled(Stat)
        self.lineEditTotal.setEnabled(Stat)
        self.lineEditLength.setEnabled(Stat)
        self.lineEditWidth.setEnabled(Stat)
        self.lineEditHeight.setEnabled(Stat)
        self.lineEdit_Niter.setEnabled(Stat)
        self.lineEdit_Error.setEnabled(Stat)


    def clear_prev(self):
        self.plotSVD.canvas.ax.clear()
        self.plotSVD.canvas.draw()
        self.plotInitSpec.canvas.ax.clear()
        self.plotInitSpec.canvas.draw()
        self.plotPurestConc.canvas.ax.clear()
        self.plotPurestConc.canvas.draw()
        self.plotPurestSpectra.canvas.ax.clear()
        self.plotPurestSpectra.canvas.draw()
        self.plot_whitelight.load(None)
        self.lineEditStatus.setText('')

        self.insp = [0]
        self.copt = [0]
        self.sopt = [0]
        self.projection = 0
        self.xplot = [0]
        self.splot = [0]
        self.u = 0
        self.s = 0
        self.v = 0
        self.dn = 0
        self.sx = 0
        self.sy = 0
        self.p = 0
        self.wavenumber = 0


    def killer_renew(self):
        self.plotPurestConc.canvas.ax.clear()
        self.plotPurestConc.canvas.draw()
        self.plotPurestSpectra.canvas.ax.clear()
        self.plotPurestSpectra.canvas.draw()
        self.lineEditStatus.setText('STOP')


#----------------------------------------------------------------------
class Multiple_Calculation(QThread):

    DataInit = pyqtSignal(int, str)
    purest = pyqtSignal(np.int,np.float64,str,np.ndarray, np.ndarray)

    QThread.setTerminationEnabled()
    def __init__(self, filenames, nr, f, max_iter, stopping_error, init,win, pol,implement, parent=None):
        QThread.__init__(self, parent)
        self.foldername = dirname(filenames[0])
        self.nr = nr
        self.max_iter = max_iter
        self.stopping_error = stopping_error
        self.init = init
        self.f = f
        self.max_iter = max_iter
        self.rem = False
        self.win = win
        self.pol = pol
        self.implement = implement
        self.filenames = filenames
        if self.init == 0:
            inguess = 'Spectra'
        else:
            inguess = 'Concentration'

        print(self.foldername)
#----------------------------------------------------------------------------------------
        now = datetime.now()
        date_time = now.strftime("%m/%d/%Y, %H:%M:%S")
        newformat = now.strftime('%m-%d-%Y_%H-%M-%S')
        self.logfile = open(self.foldername+"//logfile_"+newformat+".txt", "w")
        now = datetime.now()
        date_time = now.strftime("%m/%d/%Y, %H:%M:%S")
        self.logfile.write("%s\n" % ("Octavvs Project"))
        self.logfile.write("%s\n" % ("Multivariate Curve Resolution-Alternating Least Square"))
        self.logfile.write("%s\n" % ("Logging started at "+date_time))
        self.logfile.write("%s\n" % ("   Folder: "+self.foldername))
        self.logfile.write("%s\n" % ("   Number of components: "+str(self.nr)))
        self.logfile.write("%s\n" % ("   Noise in SIMPLISMA: "+str(self.f)))
        self.logfile.write("%s\n" % ("   Tolerance: "+str(self.stopping_error)))
        self.logfile.write("%s\n" % ("   Initial Guess: "+ inguess))
        self.logfile.write("%s\n" % ("-------------------------------------------------------"))
#----------------------------------------------------------------------------------------
    def stop(self):
        self.rem = True
        try:
            self.logfile.write("%s\n" % ("calculation was terminated"))
            self.logfile.write("%s\n" % ("-------------------------------------------------------"))
            self.logfile.close()
        except:
            pass

    def logreport(self,filenumber,filename,niter,status,maxfile):
        now = datetime.now()
        date_time = now.strftime("%m/%d/%Y, %H:%M:%S")
        self.logfile.write("%s %s %s %s\n" % (int(filenumber+1),date_time, basename(filename), status+" at "+str(niter)))


        if (filenumber+1) == maxfile:
            self.logfile.close()


    def finished_one(self, filenumber,filename, niter, status, maxfile):
        if status == 'converged' or status =='Max iterations reached':
            self.logreport(filenumber,filename,niter,status,maxfile)


    def solve_lineq(self,A,B):
        if B.ndim ==2:
            N = B.shape[-1]
        else:
            N = 0

        X_ = np.zeros((A.shape[-1], N))
        residual_ = np.zeros((N))

        if N == 0:
            X_, residual_ = nnls(A, B)
        else:
            for num in range(N):
                X_[:, num],residual_[num] = nnls(A, B[:, num])
        return X_

    def normalize(self,dn):
        nrow, ncol = dn.shape
        dauxt = np.zeros((ncol,nrow))
        aux=dn.T
        for i in range(0,ncol):
            dauxt[i,:]=aux[i,:]/np.sqrt(np.sum(aux[i,:]*aux[i,:]))
        return dauxt


    def constraint_norm(self,A):
        A = A.copy()
        A /= A.sum(axis=0)[None, :]
        return A*(A > 0)

    def run(self):
        self.maxfile = len(self.filenames)

        self.count = 0
        for i in range(0,self.maxfile):
            if self.rem:
                status = 'STOP'
                # self.purest.emit(iter,per,status, Cf.T,Sf.T)
                break
            else:

                self.filename = self.filenames[i]
                self.count = self.count + 1
                self.sx, self.sy, self.p ,self.wavenumber, self.spo = ff.readmat(self.filename)
            # self,xplot,splot,sx,sy, p, wavenumber, sp, count
                if self.implement == 1:
                    self.sp = sc.signal.savgol_filter(self.spo.T,self.win, polyorder = self.pol,deriv=1)
                    self.sp = self.sp.T

                elif self.implement == 2:
                    self.sp= sc.signal.savgol_filter(self.spo.T,self.win, polyorder = self.pol,deriv=2)
                    self.sp= self.sp.T
                else:
                    self.sp = self.spo.copy()

                self.sp = mc.nonnegative(self.sp)


                if self.init == 0:
                    insp, points = ff.initi_simplisma(self.sp,self.nr,self.f)
                    C_ = None
                    ST_ = insp
                elif self.init == 1:
                    insp, points = ff.initi_simplisma(self.sp.T,self.nr,self.f)
                    C_ = insp
                    ST_ = None

                u,s,v = np.linalg.svd(self.sp)

                self.DataInit.emit(self.count,self.filename)

                nrow, ncol = np.shape(self.sp)



                s = sc.linalg.diagsvd(s,nrow, ncol)

                u = u[:,0:self.nr]
                s = s[0:self.nr,0:self.nr]
                v = v[0:self.nr,:]
                dn = u @ s @ v


                dauxt = self.normalize(dn)

                for num in range(self.max_iter):
                    iter = num + 1
                    if self.rem:
                        status = 'STOP'
                # self.purest.emit(iter,per,status, Cf.T,Sf.T)
                        break
                    else:

                        if ST_ is not None:
                            Ctemp = self.solve_lineq(ST_.T,dauxt.T)
                            #Constraint
                            # Ctemp = constraint_norm(Ctemp)

                            Dcal = np.dot(Ctemp.T,ST_)
                            error = msea(dauxt,Dcal)

                            if iter ==1:
                                error0 = error.copy()
                                C_ = Ctemp

                            per = 100*abs(error-error0)/error
                            # print(iter,'and',per)
                            if per == 0:
                                Cf = C_.copy()
                                Sf = ST_.copy()
                                ST_temp = ST_.copy()
                                status = 'Iterating'
                                self.purest.emit(iter,per,status, Cf.T,Sf.T)

                            elif  per < self.stopping_error:
                                Cf = C_.copy()
                                Sf = ST_.copy()
                                # Save_data(Cf,Sf)
                                status = 'converged'
                                self.purest.emit(iter,per,status, Cf.T,Sf.T)
                                self.finished_one(i,self.filename,iter,status,self.maxfile)
                                break
                            else:
                                error0 = error.copy()
                                C_ = Ctemp.copy()
                                Sf = ST_.copy()
                                status = 'Iterating'
                                self.purest.emit(iter,per,status, C_.T,Sf.T)

                        if C_ is not None:
                            ST_temp = self.solve_lineq(C_.T,dauxt)
                            #Constraint
                            # ST_temp = constraint_norm(ST_temp)

                            Dcal = np.dot(C_.T,ST_temp)
                            error = msea(dauxt,Dcal)

                            if iter ==1:
                                error0 = error.copy()
                                # C_ = Ctemp
                                ST_ = ST_temp

                            per = 100*abs(error-error0)/error
                            # print(iter,'and',per)
                            if per == 0:
                                Cf = C_.copy()
                                Sf = ST_.copy()
                                status = 'Iterating'
                                self.purest.emit(iter,per,status, Cf.T,Sf.T)

                            elif per < self.stopping_error:
                                Cf = C_.copy()
                                Sf = ST_.copy()
                                # Save_data(Cf,Sf)
                                status = 'converged'
                                self.purest.emit(iter,per,status, Cf.T,Sf.T)
                                self.finished_one(i,self.filename,iter,status,self.maxfile)
                                break

                            else:
                                error0 = error.copy()
                                ST_ = ST_temp.copy()
                                status = 'Iterating'
                                self.purest.emit(iter,per,status, C_.T,ST_.T)

                        if iter == self.max_iter:
                            status = 'Max iterations reached'
                            self.purest.emit(iter,per,status, C_.T,ST_.T)
                            self.finished_one(i,self.filename,iter,status,self.maxfile)
                            break



 #----------------------------------------------------------------------


class single_report(QThread):
    purest = pyqtSignal(np.int,np.float64,str,np.ndarray, np.ndarray)
    def __init__(self, sp, nr, f, niter, stopping_error, init, parent=None
                ):
        QThread.__init__(self, parent)
        """
        Multivariate Curve Resolution - Alternating Least Square
        """
        self.sp = sp
        self.nr = nr
        self.f = f
        self.niter = niter
        self.stopping_error = stopping_error
        self.init = init
        self.rem = False

    def stop(self):
        self.rem = True


    def solve_lineq(self,A,B):
        if B.ndim ==2:
            N = B.shape[-1]
        else:
            N = 0

        X_ = np.zeros((A.shape[-1], N))
        residual_ = np.zeros((N))

        # nnls is Ax = b; thus, need to iterate along
        if N == 0:
            X_, residual_ = nnls(A, B)
        else:
            for num in range(N):
                X_[:, num],residual_[num] = nnls(A, B[:, num])
        return X_

    def normalize(self,dn):
        nrow, ncol = dn.shape
        dauxt = np.zeros((ncol,nrow))
        aux=dn.T
        for i in range(0,ncol):
            dauxt[i,:]=aux[i,:]/np.sqrt(np.sum(aux[i,:]*aux[i,:]))
        return dauxt


    def constraint_norm(self,A):
        A = A.copy()
        A /= A.sum(axis=0)[None, :]
        return A*(A > 0)


    def run(self):
        if self.init == 0:
            insp, points = ff.initi_simplisma(self.sp,self.nr,self.f)
            C_ = None
            ST_ = insp
        elif self.init == 1:
            insp, points = ff.initi_simplisma(self.sp.T,self.nr,self.f)
            C_ = insp
            ST_ = None

        u,s,v = np.linalg.svd(self.sp)
        nrow, ncol = np.shape(self.sp)
        # nr =
        s = sc.linalg.diagsvd(s,nrow, ncol)
        u = u[:,0:self.nr]
        s = s[0:self.nr,0:self.nr]
        v = v[0:self.nr,:]
        dn = u @ s @ v

        dauxt = self.normalize(dn)

        for num in range(self.niter):
            iter = num + 1
            if self.rem:
                status = 'STOP'
                # self.purest.emit(iter,per,status, Cf.T,Sf.T)
                break
            else:

                if ST_ is not None:
                    Ctemp = self.solve_lineq(ST_.T,dauxt.T)
                    #Constraint
                    # Ctemp = constraint_norm(Ctemp)

                    Dcal = np.dot(Ctemp.T,ST_)
                    error = msea(dauxt,Dcal)

                    if iter ==1:
                        error0 = error.copy()
                        C_ = Ctemp

                    per = 100*abs(error-error0)/error
                    print(iter,'and',per)
                    if per == 0:
                        Cf = C_.copy()
                        Sf = ST_.copy()
                        ST_temp = ST_.copy()
                        status = 'Iterating'
                        self.purest.emit(iter,per,status, Cf.T,Sf.T)
                    elif  per < self.stopping_error:
                        Cf = C_.copy()
                        Sf = ST_.copy()
                    # Save_data(Cf,Sf)
                        status = 'converged'
                        self.purest.emit(iter,per,status, Cf.T,Sf.T)
                        break
                    else:
                        error0 = error.copy()
                        C_ = Ctemp.copy()
                        Sf = ST_.copy()
                        # status = 'Iterating'
                        # self.purest.emit(iter,per,status, C_.T,Sf.T)

                if C_ is not None:
                    ST_temp = self.solve_lineq(C_.T,dauxt)
                    #Constraint
                    # ST_temp = constraint_norm(ST_temp)

                    Dcal = np.dot(C_.T,ST_temp)
                    error = msea(dauxt,Dcal)

                    if iter ==1:
                        error0 = error.copy()
                        # C_ = Ctemp
                        ST_ = ST_temp

                    per = 100*abs(error-error0)/error
                    print(iter,'and',per)
                    if per == 0:
                        Cf = C_.copy()
                        Sf = ST_.copy()
                        status = 'Iterating'
                        self.purest.emit(iter,per,status, Cf.T,Sf.T)

                    elif per < self.stopping_error:
                        Cf = C_.copy()
                        Sf = ST_.copy()
                    # Save_data(Cf,Sf)
                        status = 'converged'
                        self.purest.emit(iter,per,status, Cf.T,Sf.T)
                        break

                    else:
                        error0 = error.copy()
                        ST_ = ST_temp.copy()
                        status = 'Iterating'
                        self.purest.emit(iter,per,status, C_.T,ST_.T)

                if iter == self.niter:
                    status = 'Max iterations reached'
                    self.purest.emit(iter,per,status, C_.T,ST_.T)
                    break
            # Save_data(Cf,Sf)


def main():
    parser = argparse.ArgumentParser(
            description='Graphical application for MCR-ALS analysis of hyperspectral data.')
    MyMainWindow.run_octavvs_application(parser=parser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 29 21:07:55 2020

@author: carl
"""

import os
import sys
import traceback
import multiprocessing

from PyQt5.QtWidgets import QMainWindow, QApplication, QFileDialog
from PyQt5.QtWidgets import QErrorMessage, QInputDialog, QDialog, QMessageBox
from PyQt5.QtCore import Qt, QSettings
from PyQt5.Qt import qApp

from .exceptiondialog import ExceptionDialog
from .copyfigure import add_clipboard_to_figures




class OctavvsMainWindow(QMainWindow):

    fileOptions = QFileDialog.Options() | QFileDialog.DontUseNativeDialog

    octavvs_version = 'v0.1.11'

    def __init__(self, parent=None):
        super().__init__(parent)
        self.settings = QSettings('MICCS', 'OCTAVVS ' + self.program_name())
        qApp.installEventFilter(self)
        self.setupUi(self)

        self.errorMsg = QErrorMessage(self)
        self.errorMsg.setWindowModality(Qt.WindowModal)

    def post_setup(self):
        "Called by children after setting up UI but before loading data etc"
        self.setWindowTitle('OCTAVVS %s %s' % (self.program_name(), self.octavvs_version))
        ExceptionDialog.install(self)

    @classmethod
    def program_name(cls):
        "Return the name of the program that this main window represents"
        return cls.__name__

    def showDetailedErrorMessage(self, err, details):
        "Show an error dialog with details"
        q = QMessageBox(self)
        q.setIcon(QMessageBox.Critical)
        q.setWindowTitle("Error")
        q.setText(err)
        q.setTextFormat(Qt.PlainText)
        q.setDetailedText(details)
        q.addButton('OK', QMessageBox.AcceptRole)
        return q.exec()

    def loadErrorBox(self, file, err, warning=False):
        "Prepare an error dialog for failure to load a file"
        q = QMessageBox(self)
        if warning:
            q.setIcon(QMessageBox.Information)
            q.setWindowTitle("Warning loading file")
            q.setText("Warning from loading '"+file+"':\n"+err[0])
        else:
            q.setIcon(QMessageBox.Warning)
            q.setWindowTitle("Error loading file")
            q.setText("Failed to load '"+file+"':\n"+err[0])
        q.setTextFormat(Qt.PlainText)
        q.setDetailedText(err[1])
        return q

    def getLoadSaveFileName(self, title, filter=None, settingname=None,
                            savesuffix=None, multiple=False,
                            settingdefault=None,
                            directory=None, defaultfilename=None):
        "Show a file dialog and select one or more files"
        setting = self.settings.value(settingname, settingdefault
                                      ) if settingname is not None else None
        if directory is None:
            directory = setting if type(setting) is str else None
        dialog = QFileDialog(parent=self, caption=title,
                             directory=directory, filter=filter)
        if defaultfilename is not None:
            dialog.selectFile(defaultfilename)
#        if setting and type(setting) is not str:
#            dialog.restoreState(setting)
        dialog.setOption(QFileDialog.DontUseNativeDialog, True)
        if savesuffix is not None:
            dialog.setAcceptMode(QFileDialog.AcceptSave)
            dialog.setDefaultSuffix(savesuffix)
        elif multiple:
            dialog.setFileMode(QFileDialog.ExistingFiles)
        else:
            dialog.setFileMode(QFileDialog.ExistingFile)
        dialog.exec()
        files = dialog.selectedFiles()
        if not dialog.result() or not files:
            return None
        if settingname is not None:
            self.settings.setValue(settingname, os.path.dirname(files[0]))
        return files if multiple else files[0]

    def getSaveFileName(self, title, suffix='', **kwargs):
        "Show a file dialog and select an output file"
        return self.getLoadSaveFileName(
            title=title, savesuffix=suffix, **kwargs)

    def getLoadFileName(self, title, **kwargs):
        "Show a file dialog and select an input file"
        return self.getLoadSaveFileName(title=title, **kwargs)

    def getLoadFileNames(self, title, **kwargs):
        "Show a file dialog and select an input file"
        return self.getLoadSaveFileName(title=title, multiple=True, **kwargs)

    def getImageFileName(self, title, **kwargs):
        "Show a file dialog and select a single image file"
        return self.getLoadSaveFileName(title=title,
                filter="Image files (*.jpg *.jpeg *.png *.tif "
                    "*.tiff *.bmp *.gif);;All files (*)",
                **kwargs)

    def getDirectoryName(self, title, settingname=None, savesetting=True,
                         default=None):
        "Show a file dialog and select a directory"
        directory = self.settings.value(settingname, default) if \
            settingname is not None else None
        dialog = QFileDialog(parent=self, caption=title,
                             directory=directory)

        dialog.setAcceptMode(QFileDialog.AcceptSave)
        dialog.setFileMode(QFileDialog.Directory)
        dialog.setOption(QFileDialog.DontUseNativeDialog, True)
        dialog.setOption(QFileDialog.ShowDirsOnly, True)
        dialog.exec()
        dirs = dialog.selectedFiles()
        if not dialog.result() or not dirs:
            return None
        if savesetting:
            self.settings.setValue(settingname, dirs[0])
        return dirs[0]

    def updateWavenumberRange(self):
        "For derived classes, when the wn range is updated"
        pass

    def updateDimensions(self, wh):
        "For derived classes, when image width/height is updated"
        pass


    @classmethod
    def run_octavvs_application(windowclass, parser=None, parameters=[],
                                isChild=False):
        res = 1
        try:
            progver = 'OCTAVVS %s %s' % (windowclass.program_name(),
                                         OctavvsMainWindow.octavvs_version)
            windowparams = {}
            if parser is not None:
                parser.add_argument('--version', action='version',
                                    version=progver)
                selmp = multiprocessing.get_start_method(allow_none=True) is None
                if selmp:
                    parser.add_argument('--mpmethod', help='')
                args = parser.parse_args()
                windowparams = { k: args.__dict__[k] for k in parameters }
                if selmp and args.mpmethod:
                    multiprocessing.set_start_method(args.mpmethod)

            app = QApplication.instance()
            if not app:
                app = QApplication(sys.argv)
            add_clipboard_to_figures()
            window = windowclass(**windowparams)
            window.show()
            if not isChild:
                qApp.lastWindowClosed.connect(qApp.quit);
                res = app.exec_()

        except Exception:
            traceback.print_exc()
            print('Press some key to quit')
            input()
        if not isChild :
            sys.exit(res)




import sys
import csv
import glob
import os
import pandas as pd
import collections
import traceback
from os.path import basename, dirname
from pkg_resources import resource_filename
import argparse
import io
#from PyQt5.QtCore import *
#from PyQt5.QtGui import QFileDialog
#from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QApplication, QFileDialog, QMessageBox, QDialog
from PyQt5.Qt import QMainWindow, qApp #, QTimer
#from PyQt5.QtCore import (QCoreApplication, QObject, QRunnable, QThread, QThreadPool,pyqtSignal, pyqtSlot)
from PyQt5 import uic
from PyQt5.QtCore import pyqtSignal, pyqtSlot

import numpy as np
from sklearn.cluster import MiniBatchKMeans, KMeans
from sklearn.preprocessing import StandardScaler
from scipy.signal import savgol_filter
import pickle
import matplotlib
matplotlib.use('QT5Agg')
import matplotlib.pyplot as plt
from PyQt5.QtWidgets import QTableWidgetItem, QTableWidgetSelectionRange

from octavvs.mcr import ftir_function as ff
from octavvs.algorithms import correction as mc
from octavvs.ui import (FileLoader, ImageVisualizer, OctavvsMainWindow, NoRepeatStyle, uitools)

Ui_MainWindow = uic.loadUiType(resource_filename(__name__, "mcr/clustering_ui.ui"))[0]
Ui_table = uic.loadUiType(resource_filename(__name__, "mcr/table_ui.ui"),
                          from_imports=True, import_from='octavvs')[0]
Ui_DialogAbout = uic.loadUiType(resource_filename(__name__, "mcr/about.ui"),
                                from_imports=True, import_from='octavvs')[0]

class DialogAbout(QDialog, Ui_DialogAbout):
    def __init__(self, parent=None):
        QDialog.__init__(self, parent)
        qApp.installEventFilter(self)
        self.setupUi(self)


class Table(QMainWindow, Ui_table):
    def __init__(self, parent=None):
        super(Table, self).__init__(parent=None)
        qApp.installEventFilter(self)
        self.setupUi(self)
        self.actionSelect_all.triggered.connect(self.select)
        self.actionDeselect_all.triggered.connect(self.deselect)
        self.actionCopy.triggered.connect(self.copySelection)
        self.actionSave_as.triggered.connect(self.saveas)
        self.actionClose.triggered.connect(self.close)


    @pyqtSlot(np.ndarray, int, list)
    def main_data(self,data, decision,label):
        self.data_tab = data
        self.decision = decision
        self.row  = len(self.data_tab)
        self.col = len(self.data_tab.T)
        self.label = label

        if self.decision == 0:
            name = range(1,self.col)
            name = (list(map(str,name)))

            head = ['Wavenumber']
            head = head + name
            self.tableWidget.setHorizontalHeaderLabels(head)
            self.head = head

        if self.label != ['0']:
            head = ['Wavenumber']
            head = head + self.label
            self.tableWidget.setHorizontalHeaderLabels(head)
            self.head = head

        if self.data_tab is not None:
            self.row  = len(self.data_tab)
            self.col = len(self.data_tab.T)
            self.tableWidget.setRowCount(self.row)
            self.tableWidget.setColumnCount(self.col)

            for i in range(0,self.row):
                for j in range(0,self.col):
                    self.tableWidget.setItem(i, j, QTableWidgetItem(str(self.data_tab[i,j])))

    def select(self):
        self.tableWidget.setRangeSelected(QTableWidgetSelectionRange(0,0, self.row-1, self.col-1), True)

    def deselect(self):
        self.tableWidget.setRangeSelected(QTableWidgetSelectionRange(0,0, self.row-1, self.col-1), False)

    def close(self):
        self.hide()

    def copySelection(self):
        selection = self.tableWidget.selectedIndexes()
        if selection:
            rows = sorted(index.row() for index in selection)
            columns = sorted(index.column() for index in selection)
            rowcount = rows[-1] - rows[0] + 1
            colcount = columns[-1] - columns[0] + 1
            table = [[''] * colcount for _ in range(rowcount)]
            for index in selection:
                row = index.row() - rows[0]
                column = index.column() - columns[0]
                table[row][column] = index.data()
            stream = io.StringIO()
            csv.writer(stream).writerows(table)
            qApp.clipboard().setText(stream.getvalue())

    def saveas(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        filesave, _ = QFileDialog.getSaveFileName(self,"Save the data","","csv file (*.csv)", options=options)
        if filesave:
            __ , ext = os.path.splitext(filesave)
            if  ext == '.csv':
                filesave = filesave
            else:
                filesave = filesave+'.csv'
            df = pd.DataFrame(self.data_tab,columns=[self.head])
            df.to_csv(filesave, index=False)





class MyMainWindow(OctavvsMainWindow, Ui_MainWindow):
    DataUpdated = pyqtSignal(np.ndarray, int, list)
    SearchUp = pyqtSignal(str)
    Onefile = pyqtSignal(str)
    ProjUp = pyqtSignal()
    VisUp = pyqtSignal()
    ClusUp = pyqtSignal()

    @classmethod
    def program_name(cls):
        "Return the name of the program that this main window represents"
        return 'Clustering'

    def __init__(self,parent=None):
        super().__init__(parent)
        
        self.default_dir =  os.path.dirname(__file__) + "\\mcr\\"+'dat.initi'
        self.pushButtonLoadSpec.clicked.connect(self.Load_chose)
        self.lock_un(False)
        self.pushButtonExpandSpectra.clicked.connect(self.ExpandSpectra)
        self.pushButtonExpandProjection.clicked.connect(self.ExpandProjection)
        self.comboBoxCmaps.currentIndexChanged.connect(self.coloring)
        self.comboBoxMethod.currentIndexChanged.connect(self.ImageProjection)
        self.pushButtonLoadConc.clicked.connect(self.LoadPurest)
        self.horizontalSliderWavenumber.valueChanged.connect(self.Wavenumbercal)
        self.pushButtonCluster.clicked.connect(self.ClusteringCal)
        self.comboBoxVisualize.currentIndexChanged.connect(self.Cvisualize)
        self.pushButtonExpandVisual.clicked.connect(self.ExpandVis)
        self.pushButtonExpandAve.clicked.connect(self.ExpandAve)
        self.spinBoxNcluster.valueChanged.connect(self.Nclus_on)
        self.pushButtonExpandCluster.clicked.connect(self.ExpandCluster)
        self.pushButtonReduce.clicked.connect(self.Reduce)
        self.pushButtonSaveSpectra.clicked.connect(self.SaveAverage)
        self.pushButtonLoadWhite.clicked.connect(self.IMG)
        self.pushButtonRefresh.clicked.connect(self.Refresh)
        self.pushButtonNext.clicked.connect(self.Next)
        self.pushButtonPrevious.clicked.connect(self.Previous)
        self.pushButtonSC.clicked.connect(self.SC)
        self.lineEditHeight.returnPressed.connect(self.ValidationX)
        self.lineEditWidth.returnPressed.connect(self.ValidationY)
        self.pushButtonTable.clicked.connect(self.TableShow)
        self.pushButtonTableAve.clicked.connect(self.TableAve)
        self.Tableview = Table(self)

        self.About = DialogAbout()

        self.actionAbout.triggered.connect(self.About.show)
        self.Tableview.actionabout.triggered.connect(self.About.show)

        self.DataUpdated.connect(self.Tableview.main_data)
        self.SearchUp.connect(self.search_whole_folder)
        self.Onefile.connect(self.readfile)
        self.ProjUp.connect(self.ImageProjection)
        self.VisUp.connect(self.Cvisualize)
        self.ClusUp.connect(self.ClusteringCal)

        self.post_setup()

        self.checkBoxinvert.toggled.connect(self.Inverting)
        
        self.comboBoxImp.currentIndexChanged.connect(self.Select_spectra)
        self.spinBoxWlength.valueChanged.connect(self.Select_spectra)
        self.spinBoxPoly.valueChanged.connect(self.Select_spectra)


    def closeEvent(self, event):
        msgBox = QMessageBox()
        msgBox.setIcon(QMessageBox.Question)
        msgBox.setText("Warning")
        msgBox.setInformativeText('Are you sure to close the window ?')
        msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        msgBox.setDefaultButton(QMessageBox.No)
        reply = msgBox.exec_()
        if reply == QMessageBox.Yes:
            self.Tableview.close()
            self.About.close()
            plt.close('all')
#            qApp.quit()
        else:
            event.ignore()

    def TableShow(self):
        self.VisUp.emit()
        self.Tableview.close()
        # self.VisUp.emit()
        self.Tableview.show()


    def Load_chose(self):
        try:    
            infile = open(self.default_dir,'rb')
            dire = pickle.load(infile)
            infile.close()
        except:
            dire = os.path.dirname(__file__)


        if self.comboBoxSingMul.currentIndex() == 1:
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            name, __ = QFileDialog.getOpenFileName(self,"Open Matrix File", dire,"Matrix File (*.mat)", options=options)
            self.foldername = dirname(name)
            if self.foldername:
                outfile = open(self.default_dir,'wb')
                pickle.dump(self.foldername,outfile)
                outfile.close()
                
                self.SearchUp.emit(self.foldername)
                self.pushButtonNext.setEnabled(True)
                self.pushButtonPrevious.setEnabled(True)
            else:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Critical)
                msg.setText('No file is loaded')
                msg.setInformativeText("Please select a file")
                msg.setWindowTitle("Warning")
                msg.setStandardButtons(QMessageBox.Ok )
                msg.exec_()

        elif self.comboBoxSingMul.currentIndex() == 0:
            self.LoadSpec()
            self.pushButtonNext.setEnabled(False)
            self.pushButtonPrevious.setEnabled(False)

        elif self.comboBoxSingMul.currentIndex() == 2:
             options = QFileDialog.Options()
             options |= QFileDialog.DontUseNativeDialog
             fileNames, _ = QFileDialog.getOpenFileNames(self,"Open Matrix File", dire,"Matrix File (*.mat)", options=options)
             if fileNames:
                 outfile = open(self.default_dir,'wb')
                 pickle.dump(dirname(fileNames[0]),outfile)
                 outfile.close()
                 self.foldername  = dirname(fileNames[0])

                 count = 0
                 name =  {}
                 foldername = dirname(fileNames[0])
                 for file in fileNames:
                     name[count] = file
                     count += 1
                # count = 0

                 w = csv.writer(open(foldername+"//Fileall.csv", "w"))
                 for key, val in sorted(name.items(), key=lambda item: item[1]):
#                    for key, val in sorted(name.items()):
                     w.writerow([key, val])
                     
                 self.lineEditTotal.setText(str(count))
                 self.lineEditFileNum.setText(str(1))   
                 self.pushButtonNext.setEnabled(True)
                 self.pushButtonPrevious.setEnabled(True)
                 self.Onefile.emit(name[0])
             else:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Critical)
                msg.setText('No file is loaded')
                msg.setInformativeText("Please select a file")
                msg.setWindowTitle("Warning")
                msg.setStandardButtons(QMessageBox.Ok )
                msg.exec_()



    def LoadSpec(self):
        try:    
            infile = open(self.default_dir,'rb')
            dire = pickle.load(infile)
            infile.close()
        except:
            dire = os.path.dirname(__file__)
            
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        self.fileName, __ = QFileDialog.getOpenFileName(self,"Open Matrix File", dire,"Matrix File (*.mat)")#, options=options)
        if self.fileName:
            outfile = open(self.default_dir,'wb')
            pickle.dump(dirname(self.fileName),outfile)
            outfile.close()
            
            self.foldername = dirname(self.fileName)
            self.lineEditFileNum.setText('1')
            self.lineEditTotal.setText('1')
            self.Onefile.emit(self.fileName)

        else:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setText('No file is loaded')
            msg.setInformativeText("Please select a file")
            msg.setWindowTitle("Warning")
            msg.setStandardButtons(QMessageBox.Ok )
            msg.exec_()


    def Next(self):
        if int(self.lineEditFileNum.text()) != int(self.lineEditTotal.text()):
            df = pd.read_csv(self.foldername+"//Fileall.csv",header=None)
            count = int(self.lineEditFileNum.text())
            filename = df.iloc[count,1]
            self.lineEditFileNum.setText(str(count+1))
            # self.readfile(filename)
            self.Onefile.emit(filename)
        else:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setText('No More File')
            msg.setInformativeText("Finish")
            msg.setWindowTitle("Warning")
            msg.setStandardButtons(QMessageBox.Ok )
            msg.exec_()


    def Previous(self):
        if int(self.lineEditFileNum.text()) != 1:
            df = pd.read_csv(self.foldername+"//Fileall.csv",header=None)
            count = int(self.lineEditFileNum.text())
            count -=1
            filename = df.iloc[count-1,1]
            self.lineEditFileNum.setText(str(count))
            self.Onefile.emit(filename)
        else:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setText('This is the first file')
            msg.setInformativeText("Click Next for net")
            msg.setWindowTitle("Warning")
            msg.setStandardButtons(QMessageBox.Ok )
            msg.exec_()


    def ValidationX(self):
        x = int(self.lineEditHeight.text())
        y = int(self.lineEditWidth.text())
        z = int(self.lineEditLength.text())
        xy = int(self.sx*self.sy)

        if x == 0 or y == 0:
            self.lineEditHeight.setText(str(self.sy))
            self.lineEditWidth.setText(str(self.sx))
            x = self.sx
            y = self.sy

        elif int(x*y) != xy:
            excess = np.mod(xy,x)
            if excess == 0 :
                y=xy/x
                y = int(y)
                self.lineEditWidth.setText(str(y))
            else:
                self.lineEditHeight.setText(str(self.sy))
                self.lineEditWidth.setText(str(self.sx))
                x = self.sx
                y = self.sy
        else:
            self.lineEditHeight.setText(str(x))
            self.lineEditWidth.setText(str(y))


        self.p = np.reshape(self.p,(z,x,y))
        self.ProjUp.emit()
        self.VisUp.emit()
        self.ClusUp.emit()

    def ValidationY(self):
        x = int(self.lineEditHeight.text())
        y = int(self.lineEditWidth.text())
        z = int(self.lineEditLength.text())
        xy = int(self.sx*self.sy)

        if x == 0 or y == 0:
            self.lineEditHeight.setText(str(self.sy))
            self.lineEditWidth.setText(str(self.sx))
            x = self.sx
            y = self.sy

        elif int(x*y) != xy:
            excess = np.mod(xy,y)
            if excess == 0:
                x=xy/y
                x = int(x)
                self.lineEditHeight.setText(str(x))
            else:
                self.lineEditHeight.setText(str(self.sy))
                self.lineEditWidth.setText(str(self.sx))
                x = self.sx
                y = self.sy
        else:
            self.lineEditHeight.setText(str(x))
            self.lineEditWidth.setText(str(y))


        self.p = np.reshape(self.p,(z,x,y))
        self.ProjUp.emit()
        self.VisUp.emit()
        self.ClusUp.emit()

    def SC(self):
        name = self.lineEditFilename.text()
        QApplication.primaryScreen().grabWindow(self.winId()).save(self.foldername+'/'+name+'_SC'+'.png')


    @pyqtSlot(str)
    def search_whole_folder(self, foldername):
        count = 0
        name =  {}
        a = [x[0] for x in os.walk(foldername)]
        for i in a:
            os.chdir(i)
            for file in glob.glob('*.mat'):
                name[count] = str(i+'/'+file)
                count += 1

        w = csv.writer(open(foldername+"//Fileall.csv", "w"))
        for key, val in sorted(name.items(), key=lambda item: item[1]):
            w.writerow([key, val])

        self.lineEditTotal.setText(str(count))
        self.lineEditFileNum.setText(str(1))
        self.Onefile.emit(name[0])

    @pyqtSlot(str)
    def readfile(self,fileName):
        self.img = None
        try:
            self.img = plt.imread(fileName.replace(fileName.split('.0')[-1],'.jpg'))
        except:
            pass

        try:
            self.sx,self.sy, self.pin ,self.wavenumber, self.spo = ff.readmat(fileName)
            self.spo = mc.nonnegative(self.spo)
            self.lineEditHeight.setText(str(self.sy))
            self.lineEditWidth.setText(str(self.sx))
        except:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setText('No file is loaded')
            msg.setInformativeText('File can not be opened due to the the structure of data')
            msg.setWindowTitle("Warning")
            msg.setStandardButtons(QMessageBox.Ok )
            msg.exec_()
            self.lineEditFileNum.setText('')
            self.lineEditTotal.setText('')
            pass


        
        # self.index = np.random.randint(0,int(self.sx*self.sy),(20))
        kmeans = MiniBatchKMeans(n_clusters=8, random_state=0).fit(self.spo.T)
        
        self.raw = kmeans.cluster_centers_
        self.raw = self.raw.T

        
        self.Select_spectra()
        
        self.clear_all()
        self.lock_un(True)
        self.lineEditFilename.setText((basename(fileName).replace('.mat','')))
        self.lineEditDirSpectra.setText(fileName)

        


        self.PlotSpectraSample()

        self.labelMinwn.setText(str("%.2f" % np.min(self.wavenumber)))
        self.labelMaxwn.setText(str("%.2f" % np.max(self.wavenumber)))
        self.lineEditLength.setText(str(len(self.wavenumber)))
        self.lineEditWavenumber.setText(str("%.2f" % np.min(self.wavenumber)))

 
        self.ProjUp.emit()

        if self.img is not None:
            self.plot_White.canvas.ax.clear()
            self.plot_White.canvas.ax.imshow(self.img)
            self.plot_White.canvas.fig.tight_layout()
            self.plot_White.canvas.ax.set_axis_off() 
            self.plot_White.canvas.draw()


        filepure = fileName.replace('.mat','')
        filepure = filepure+'_purest.csv'
        if os.path.isfile(filepure) == True:
             msgBox = QMessageBox()
             msgBox.setIcon(QMessageBox.Question)
             msgBox.setText("Information")
             msgBox.setInformativeText('The '+basename(filepure)+' is available are you going to use it as purest spectra ?')
             msgBox.setStandardButtons(QMessageBox.Yes| QMessageBox.No)
             msgBox.setDefaultButton(QMessageBox.No)
             reply = msgBox.exec_()
             if reply == QMessageBox.Yes:
                self.AutoLoad(filepure)
             else:
                self.LoadPurest()
        else:
            self.LoadPurest()

    def Inverting(self):
        if self.comboBoxVisualize.currentIndex() == 0:
            self.PlotSpectraSample()
        else:
            self.VisUp.emit()
            self.ClusUp.emit()

        if hasattr(self,'df_spec'):
            self.ClusUp.emit()
            if self.pushButtonReduce.isEnabled():
                pass
            else:
                self.Reduce()

    def Select_spectra(self):
        win = int(self.spinBoxWlength.value())
        pol = int(self.spinBoxPoly.value())
        if self.comboBoxImp.currentIndex() == 0:
            self.sp = self.spo.copy()
            self.sp = mc.nonnegative(self.sp)
            self.p = self.pin.copy() 
        elif self.comboBoxImp.currentIndex() == 1:
            self.sp = savgol_filter(self.spo.T,win, polyorder = pol,deriv=1)
            self.sp = self.sp.T
            self.sp = mc.nonnegative(self.sp)
            self.p = self.sp.reshape(np.shape(self.pin),order='C')
                
        elif self.comboBoxImp.currentIndex() == 2:
            self.sp = savgol_filter(self.spo.T,win, polyorder = pol,deriv=2)
            self.sp = self.sp.T
            self.sp = mc.nonnegative(self.sp)
            self.p = self.sp.reshape(np.shape(self.pin),order='C')
            
          
        try:
            x = int(self.lineEditHeight.text())
            y = int(self.lineEditWidth.text())
            z = int(self.lineEditLength.text())
            self.p = np.reshape(self.p,(z,x,y))
        except ValueError:
            self.lineEditWidth.setText(str(self.sx))
            self.lineEditHeight.setText(str(self.sy))

        self.ProjUp.emit()
        self.PlotSpectraSample()
        if hasattr(self,'df_spec'):
            self.ClusUp.emit()




    def PlotSpectraSample(self):
        win = int(self.spinBoxWlength.value())
        pol = int(self.spinBoxPoly.value())
        
        if self.comboBoxImp.currentIndex() == 0:
            self.yvis = self.raw.copy()
            self.spinBoxWlength.hide()
            self.spinBoxPoly.hide()
            
            self.spinBoxWlength.hide()
            self.spinBoxPoly.hide()
            self.labelWl.hide()
            self.labelPl.hide()

            
        elif self.comboBoxImp.currentIndex() == 1:
            self.yvis= savgol_filter(self.raw.T,win, polyorder = pol,deriv=1)
            self.yvis= self.yvis.T
            
            self.spinBoxWlength.show()
            self.spinBoxPoly.show()

            self.spinBoxWlength.show()
            self.spinBoxPoly.show()
            
            self.labelWl.show()
            self.labelPl.show()

            
        else:
            self.yvis= savgol_filter(self.raw.T,win, polyorder = pol,deriv=2)
            self.yvis= self.yvis.T
            self.labelWl.show()
            self.labelPl.show()
            self.spinBoxWlength.show()
            self.spinBoxPoly.show()


        self.DataUpdated.emit(np.column_stack((self.wavenumber,self.yvis)),0,['0'])
        
        self.yvis = mc.nonnegative(self.yvis)
        
        self.plot_specta.canvas.ax.clear()
        self.plot_specta.canvas.ax.plot(self.wavenumber,self.yvis)
        if self.checkBoxinvert.isChecked():
            self.plot_specta.Invert()
        self.plot_specta.canvas.fig.tight_layout()
        self.plot_specta.canvas.draw()
        self.ExpandSpectraU()


    def LoadPurest(self):
        sugest = self.foldername+'//'+self.lineEditFilename.text()+'_purest.csv'
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        filepurest, _ = QFileDialog.getOpenFileName(self,"Open Matrix File",sugest,"Matrix File (*.csv)", options=options)
        if filepurest:
            self.comboBoxVisualize.clear()
            self.comboBoxVisualize.addItem('Spectra and White Light Image')

            self.lineEditDirPurest.setText(filepurest)
            dfpurest = pd.read_csv(filepurest, header= None)
            
            #Old System Before implementing First or Second Derivatives
            if (int(self.lineEditLength.text())+int(self.sx*self.sy)) == len(dfpurest):
                self.compen = 0
                self.comboBoxImp.setCurrentIndex(0)
            
            #Old System after implementing First or Second Derivatives, but error
            #if  nr = 2, but mostly we never use nr = 2, but I keep this to read 
            #old data                
            elif (int(self.lineEditLength.text())+int(self.sx*self.sy)) + 1 == len(dfpurest):
                self.compen = 1
                self.spinBoxWlength.setValue(int(dfpurest.iloc[0,1]))
                self.spinBoxPoly.setValue(int(dfpurest.iloc[0,2]))
                self.comboBoxImp.setCurrentIndex(int(dfpurest.iloc[0,0]))
                self.Select_spectra()


            # New System, there is no error even we use nr = 2
            elif (int(self.lineEditLength.text())+int(self.sx*self.sy)) +3 == len(dfpurest):
                self.compen = 3
                self.spinBoxWlength.setValue(int(dfpurest.iloc[1,0]))
                self.spinBoxPoly.setValue(int(dfpurest.iloc[2,0]))
                self.comboBoxImp.setCurrentIndex(int(dfpurest.iloc[0,0]))
                self.Select_spectra()

                                 
                        
            
            self.df_spec = dfpurest.iloc[self.compen:int(self.lineEditLength.text())+self.compen,:]
            self.df_conc = dfpurest.iloc[self.compen+int(self.lineEditLength.text()):,:]
            self.ClusUp.emit()
            self.Nclus_on()
            for comp1 in range(0,len(self.df_conc.T)):
                self.comboBoxVisualize.addItem("component_"+str(comp1+1))

    def AutoLoad(self,filepurest):       
        self.comboBoxVisualize.clear()
        self.comboBoxVisualize.addItem('Spectra and White Light Image')
        self.lineEditDirPurest.setText(filepurest)
        dfpurest = pd.read_csv(filepurest, header= None)
        
        #Old System Before implementing First or Second Derivatives
        if (int(self.lineEditLength.text())+int(self.sx*self.sy)) == len(dfpurest):
            self.compen = 0
            self.comboBoxImp.setCurrentIndex(0)

        #Old System after implementing First or Second Derivatives, but error
        #if  nr = 2, but mostly we never use nr = 2, but I keep this to read 
        #old data                
        if (int(self.lineEditLength.text())+int(self.sx*self.sy)) + 1 == len(dfpurest):
            self.compen = 1
            self.spinBoxWlength.setValue(int(dfpurest.iloc[0,1]))
            self.spinBoxPoly.setValue(int(dfpurest.iloc[0,2]))
            self.comboBoxImp.setCurrentIndex(int(dfpurest.iloc[0,0]))
            self.Select_spectra()

        # New System, there is no error even we use nr = 2
        if (int(self.lineEditLength.text())+int(self.sx*self.sy)) + 3 == len(dfpurest):
            self.compen = 3
            self.spinBoxWlength.setValue(int(dfpurest.iloc[1,0]))
            self.spinBoxPoly.setValue(int(dfpurest.iloc[2,0]))
            self.comboBoxImp.setCurrentIndex(int(dfpurest.iloc[0,0]))
            self.Select_spectra()
        
        
        
        self.df_spec = dfpurest.iloc[self.compen:int(self.lineEditLength.text())+self.compen,:]
        self.df_conc = dfpurest.iloc[self.compen+int(self.lineEditLength.text()):,:]

        # self.ClusteringCal()
        self.ClusUp.emit()
        self.Nclus_on()
        for comp1 in range(0,len(self.df_conc.T)):
            self.comboBoxVisualize.addItem("component_"+str(comp1+1))

    def IMG(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        fileName, _ = QFileDialog.getOpenFileName(self,"Open Image File", "",
                          "Image (*.jpg *.jpeg *.bmp *.png .tif *.tiff)", options=options)
        if fileName:
            self.img = plt.imread(fileName)
            self.plot_White.canvas.ax.clear()
            self.plot_White.canvas.ax.imshow(self.img)
            self.plot_White.canvas.fig.tight_layout()
            self.plot_White.canvas.ax.set_axis_off() 
            self.plot_White.canvas.draw()
            # self.Cvisualize()
            self.VisUp.emit()


    def ExpandCluster(self):
        fig = plt.figure("Segmentation Map", tight_layout={'pad':.5})
        fig.clear()
        ax = fig.gca()
        ax.imshow(self.mapping,cmap=self.cmap)
        plt.colorbar(ax.imshow(self.mapping,cmap=self.cmap))
        fig.show()


    def ExpandClusterU(self):
        if plt.fignum_exists("Segmentation Map"):
            fig = plt.figure("Segmentation Map", tight_layout={'pad':.5})
            fig.clear()
            ax = fig.gca()
            ax.imshow(self.mapping,cmap=self.cmap)
            plt.colorbar(ax.imshow(self.mapping,cmap=self.cmap))
            fig.canvas.draw_idle()
        else:
            pass


    @pyqtSlot()
    def ClusteringCal(self):
        nclus = int(self.spinBoxNcluster.value())
        method = int(self.comboBoxMethodCluster.currentIndex())

        if self.comboBoxNormalization.currentIndex() == 0:
            data = self.df_conc
        else:
            data = StandardScaler().fit_transform(self.df_conc)

        if method == 0:
            self.clus = KMeans(n_clusters = nclus, random_state=0).fit(data)
        else:
            self.clus = MiniBatchKMeans(n_clusters = nclus, random_state=0).fit(data)


        self.mapping = np.reshape(self.clus.labels_,(int(self.lineEditHeight.text()),
                                           int(self.lineEditWidth.text())))


        c1 =  str(self.comboBoxC1.currentText())
        c2 =  str(self.comboBoxC2.currentText())
        c3 =  str(self.comboBoxC3.currentText())
        c4 =  str(self.comboBoxC4.currentText())
        c5 =  str(self.comboBoxC5.currentText())
        c6 =  str(self.comboBoxC6.currentText())
        c7 =  str(self.comboBoxC7.currentText())
        c8 =  str(self.comboBoxC8.currentText())

        t1 = self.lineEditA1.text()
        t2 = self.lineEditA2.text()
        t3 = self.lineEditA3.text()
        t4 = self.lineEditA4.text()
        t5 = self.lineEditA5.text()
        t6 = self.lineEditA6.text()
        t7 = self.lineEditA7.text()
        t8 = self.lineEditA8.text()



        if nclus == 1:
            self.clis =[c1]
            self.label = [t1]

        elif nclus == 2:
            self.clis =[c1,c2]
            self.label = [t1,t2]

        elif nclus == 3:
            self.clis =[c1,c2,c3]
            self.label = [t1,t2,t3]

        elif nclus == 4:
            self.clis =[c1,c2,c3,c4]
            self.label = [t1,t2,t3,t4]

        elif nclus == 5:
            self.clis =[c1,c2,c3,c4,c5]
            self.label = [t1,t2,t3,t4,t5]

        elif nclus == 6:
            self.clis =[c1,c2,c3,c4,c5,c6]
            self.label = [t1,t2,t3,t4,t5,t6]

        elif nclus == 7:
            self.clis =[c1,c2,c3,c4,c5,c6,c7]
            self.label = [t1,t2,t3,t4,t5,t6,t7]

        elif nclus == 8:
            self.clis =[c1,c2,c3,c4,c5,c6,c7,c8]
            self.label = [t1,t2,t3,t4,t5,t6,t7,t8]

        if nclus > 8:
            self.cmap = str(self.comboBoxColorBig.currentText())
        else:
            self.cmap = matplotlib.colors.ListedColormap(self.clis)

        self.plotCluster.canvas.ax.clear()
        self.plotCluster.canvas.ax.imshow(self.mapping,cmap=self.cmap)
        self.plotCluster.canvas.fig.tight_layout()
        self.plotCluster.canvas.ax.set_axis_off() 
        self.plotCluster.canvas.draw()


        self.ExpandClusterU()

        self.color=[c1,c2,c3,c4,c5,c6,c7,c8]

        self.plotAverage.canvas.ax.clear()
        self.plotAverage.canvas.ax.set_prop_cycle(color=self.color)

        self.pushButtonReduce.setEnabled(True)

        indori = self.clus.labels_
        self.inde ={}
        self.spmean = np.zeros((nclus,len(self.sp)))
        for jj in range(0,nclus):
            self.inde[jj]=[i for i, e in enumerate(indori) if e == jj]
            self.spmean[jj,:]= np.mean(self.sp[:,self.inde[jj]], axis = 1)


        self.plotAverage.canvas.ax.plot(self.wavenumber, self.spmean.T )
        if self.checkBoxinvert.isChecked():
            self.plotAverage.Invert()


        if nclus <= 8:
            self.plotAverage.canvas.ax.legend(self.label,loc='best')
            self.plotAverage.canvas.fig.tight_layout()
        else:
            self.plotAverage.canvas.fig.tight_layout()

        self.ExpandAveU()   

        self.plotAverage.canvas.draw()
        anotclus=len(set(self.clis))
        self.lineEditAnotClus.setText(str(anotclus))



    def ExpandAve(self):
        fig = plt.figure("Average Spectra", tight_layout={'pad':.5})
        fig.clear()
        ax = fig.gca()
        ax.set_prop_cycle(color=self.color)
        ax.plot(self.wavenumber, self.spmean.T)
        if self.checkBoxinvert.isChecked():
            ax.invert_xaxis()
        ax.legend(self.label,loc='best')
        ax.set_xlabel("Wavenumber(1/cm)",fontsize=14)
        ax.set_ylabel("Absorption(arb. units)",fontsize=14)
        ax.tick_params(axis='both',direction='in', length=8, width=1)
        ax.tick_params(axis='both',which='major',labelsize=14)
        fig.show()

    def ExpandAveU(self):
        if plt.fignum_exists("Average Spectra"):
            fig = plt.figure("Average Spectra", tight_layout={'pad':.5})
            fig.clear()
            ax = fig.gca()
            ax.set_prop_cycle(color=self.color)
            ax.plot(self.wavenumber, self.spmean.T)
            if self.checkBoxinvert.isChecked():
                ax.invert_xaxis()
            ax.legend(self.label,loc='best')
            ax.set_xlabel("Wavenumber(1/cm)",fontsize=14)
            ax.set_ylabel("Absorption(arb. units)",fontsize=14)
            ax.tick_params(axis='both',direction='in', length=8, width=1)
            ax.tick_params(axis='both',which='major',labelsize=14)
            fig.canvas.draw_idle()    
        else:
            pass




    def ExpandSpectra(self):
        # plt.close('Spectra')
        fig = plt.figure('Spectra', tight_layout={'pad':.5})
        fig.clear()
        ax = fig.gca()
        if self.comboBoxVisualize.currentIndex() == 0:
            ax.plot(self.wavenumber,self.yvis)
            if self.comboBoxMethod.currentIndex() == 2:
                ax.axvline(x=self.wavenumv)
        else:
            ax.plot(self.wavenumber, self.datas)

        if self.checkBoxinvert.isChecked():
            ax.invert_xaxis()
        ax.set_xlabel("Wavenumber(1/cm)")#,fontsize=24)
        ax.set_ylabel("Absorption(arb. units)")#,fontsize=24)
        ax.tick_params(axis='both',direction='in', length=8, width=1)
        ax.tick_params(axis='both',which='major')#,labelsize=24)
        fig.show()

    def ExpandSpectraU(self):
        if plt.fignum_exists("Spectra"):
            fig = plt.figure("Spectra")
            ax = fig.gca()
            ax.clear()
            if self.comboBoxVisualize.currentIndex() == 0:
                ax.plot(self.wavenumber,self.yvis)
                if self.comboBoxMethod.currentIndex() == 2:
                    ax.axvline(x=self.wavenumv)
            else:
                ax.plot(self.wavenumber, self.datas)

            if self.checkBoxinvert.isChecked():
                ax.invert_xaxis()
                
            ax.set_xlabel("Wavenumber(1/cm)")#,fontsize=24)
            ax.set_ylabel("Absorption(arb. units)")#,fontsize=24)
            ax.tick_params(axis='both',direction='in', length=8, width=1)
            ax.tick_params(axis='both',which='major')#,labelsize=24)
            fig.canvas.draw_idle()
            
        else:
            pass

    def ExpandProjection(self):
        # plt.close("Image Projection")
        fig = plt.figure("Image Projection", tight_layout={'pad':.5})
        fig.clear()
        ax = fig.gca()
        ax.imshow(self.projection,str(self.comboBoxCmaps.currentText()))
        plt.colorbar(ax.imshow(self.projection,str(self.comboBoxCmaps.currentText())))
        fig.show()

    def ExpandProjU(self):
        if plt.fignum_exists("Image Projection"):
            fig = plt.figure("Image Projection")
            fig.clear()
            ax = fig.gca()
            ax.imshow(self.projection,str(self.comboBoxCmaps.currentText()))
            plt.colorbar(ax.imshow(self.projection,str(self.comboBoxCmaps.currentText())))
            fig.canvas.draw_idle()
        else:
            pass



    def coloring(self):
        self.plot_visual.canvas.ax.clear()
        self.plot_visual.canvas.ax.imshow(self.projection,str(self.comboBoxCmaps.currentText()))
        self.plot_visual.canvas.fig.tight_layout()
        self.plot_visual.canvas.ax.set_axis_off() 
        self.plot_visual.canvas.draw()
        self.VisUp.emit()
        self.ExpandProjU()


    @pyqtSlot()
    def ImageProjection(self):
        if  self.comboBoxMethod.currentIndex() == 0:
            self.lineEditWavenumber.setEnabled(False)
            self.horizontalSliderWavenumber.setEnabled(False)
            self.projection = ff.proarea(self.p,self.wavenumber)
        if  self.comboBoxMethod.currentIndex() == 1:
            self.lineEditWavenumber.setEnabled(False)
            self.horizontalSliderWavenumber.setEnabled(False)
            self.projection = ff.promip(self.p)
        if  self.comboBoxMethod.currentIndex() == 2:
            self.lineEditWavenumber.setEnabled(True)
            self.horizontalSliderWavenumber.setEnabled(True)
            self.wavenumv = float(self.lineEditWavenumber.text())
            self.projection = ff.prowavenum(self.p,self.wavenumber,self.wavenumv)

        self.plot_visual.canvas.ax.clear()
        self.plot_visual.canvas.ax.imshow(self.projection,str(self.comboBoxCmaps.currentText()))
        self.plot_visual.canvas.fig.tight_layout()
        self.plot_visual.canvas.ax.set_axis_off() 
        self.plot_visual.canvas.draw()

        self.ExpandProjU()
        self.ExpandSpectraU()

    def Wavenumbercal(self):
        nnow1 = ((np.max(self.wavenumber) - np.min(self.wavenumber))
        *float(self.horizontalSliderWavenumber.value())/10000.0 + np.min(self.wavenumber))
        nnow1 = "%.2f" % nnow1
        self.lineEditWavenumber.setText(str(nnow1))
        self.wavenumv = float(self.lineEditWavenumber.text())
        self.projection = ff.prowavenum(self.p,self.wavenumber,self.wavenumv)

        self.PlotSpectraSample()

        self.plot_visual.canvas.ax.clear()
        self.plot_visual.canvas.ax.imshow(self.projection,str(self.comboBoxCmaps.currentText()))
        self.plot_visual.canvas.fig.tight_layout()
        self.plot_visual.canvas.ax.set_axis_off() 
        self.plot_visual.canvas.draw()

        self.ExpandProjU()
        self.ExpandSpectraU()


    @pyqtSlot()
    def Cvisualize (self):
        if self.comboBoxVisualize.currentIndex() == 0:
            if self.img is not None:
                self.plotMultiVisual.canvas.ax.clear()
                self.plotMultiVisual.canvas.ax.imshow(self.img)
                self.plotMultiVisual.canvas.fig.tight_layout()
                self.plotMultiVisual.canvas.ax.set_axis_off() 
                self.plotMultiVisual.canvas.draw()

            self.PlotSpectraSample()
            # self.plot_specta.canvas.ax.clear()
            # self.plot_specta.canvas.ax.plot(self.wavenumber,self.sp[:,self.index])
            # self.plot_specta.canvas.fig.tight_layout()
            # self.plot_specta.canvas.draw()
            self.DataUpdated.emit(np.column_stack((self.wavenumber,self.yvis)),0,['0'])

        if 'component_' in self.comboBoxVisualize.currentText():
            import re
            val = int(re.search(r'\d+', self.comboBoxVisualize.currentText()).group()) - 1
            self.datap = self.df_conc.iloc[:,val].to_numpy()

            self.component = np.reshape(self.datap,(int(self.lineEditHeight.text()),
                                           int(self.lineEditWidth.text())))
            self.plotMultiVisual.canvas.ax.clear()
            self.plotMultiVisual.canvas.ax.imshow(self.component,str(self.comboBoxCmaps.currentText()))
            self.plotMultiVisual.canvas.fig.tight_layout()
            self.plotMultiVisual.canvas.ax.set_axis_off() 
            self.plotMultiVisual.canvas.draw()

            self.datas = self.df_spec.iloc[:,val].to_numpy()
            self.plot_specta.canvas.ax.clear()
            self.plot_specta.canvas.ax.plot(self.wavenumber, self.datas)
            if self.checkBoxinvert.isChecked():
                self.plot_specta.Invert()
            self.plot_specta.canvas.fig.tight_layout()
            self.plot_specta.canvas.draw()


            # print(len(self.datas))
            self.DataUpdated.emit(np.column_stack((self.wavenumber,self.datas)),0,[self.comboBoxVisualize.currentText()])


        self.ExpandSpectraU()
        self.ExpandVisU()


    def ExpandVis(self):
        # plt.close('Image Projection')
        fig = plt.figure('Image Visualization', tight_layout={'pad':.5})
        fig.clear()
        ax = fig.gca()
        if self.comboBoxVisualize.currentIndex() == 0:
            ax.imshow(self.img)
        else:
            ax.imshow(self.component,str(self.comboBoxCmaps.currentText()))
            plt.colorbar(ax.imshow(self.component,str(self.comboBoxCmaps.currentText())))
        fig.show()


    def ExpandVisU(self):
        if plt.fignum_exists('Image Visualization'):
            fig = plt.figure('Image Visualization')
            fig.clear()
            ax = fig.gca()
            if self.comboBoxVisualize.currentIndex() == 0:
                ax.imshow(self.img)
            else:
                ax.imshow(self.component,str(self.comboBoxCmaps.currentText()))
                plt.colorbar(ax.imshow(self.component,str(self.comboBoxCmaps.currentText())))
            fig.canvas.draw_idle()
        else:
            pass



    def Reduce(self):
        anotclus=len(set(self.clis))
        self.lineEditAnotClus.setText(str(anotclus))

        if self.spinBoxNcluster.value() != anotclus:
            sim = [item for item, count in collections.Counter(self.clis).items() if count > 1]
            ind = ff.listindexes(self.clis)
            new = np.zeros((len(self.wavenumber),len(sim)))
            suma = np.zeros((len(sim)))


            olddict = dict(zip(self.clis,self.spmean))
            for ii in range(0,len(sim)):
                for jj in range(len(ind[sim[ii]])):
                    auin = int(ind[sim[ii]][jj])
                    suma[ii] = int(suma[ii]) + len(self.inde[auin])
                    new[:,ii] = new[:,ii] + self.spmean[auin,:]*len(self.inde[auin])
                new[:,ii]/= suma[ii]

            #Color of new and multiple spectra
            nosim = [item for item, count in collections.Counter(self.clis).items() if count > 0]


            count = 0
            new_col = 0
            new_col = sim
            if len(sim) != anotclus:
                single = list(set(nosim)-set(sim))
                dumy = np.zeros((len(self.wavenumber),len(single)))
                for jj in single:
                    dumy[:,count] = olddict[jj]
                    count+=1
                    new_col.append(jj)
                new = np.concatenate((new,dumy), axis = 1)

            else:
                new_label =     [item for item, count in collections.Counter(self.label).items() if count > 0]

            new_label=[]
            for mem in new_col:
                index = self.color.index(mem)
                new_label.append(self.label[index])



            self.spmean = new.T
            self.label = new_label
            self.color = new_col

            self.plotAverage.canvas.ax.clear()
            self.plotAverage.canvas.ax.set_prop_cycle(color=self.color)
            self.plotAverage.canvas.ax.plot(self.wavenumber, self.spmean.T )
            self.plotAverage.canvas.ax.legend(self.label,loc= 'best')
            if self.checkBoxinvert.isChecked():
                self.plotAverage.Invert()
            self.plotAverage.canvas.fig.tight_layout()
            self.plotAverage.canvas.draw()

            self.ExpandAveU()
            self.pushButtonReduce.setEnabled(False)

    def Nclus_on(self):
        nclus = int(self.spinBoxNcluster.value())

        if nclus == 1:
            self.comboBoxC1.setEnabled(True)
            self.comboBoxC2.setEnabled(False)
            self.comboBoxC3.setEnabled(False)
            self.comboBoxC4.setEnabled(False)
            self.comboBoxC5.setEnabled(False)
            self.comboBoxC6.setEnabled(False)
            self.comboBoxC7.setEnabled(False)
            self.comboBoxC8.setEnabled(False)

            self.lineEditA1.setEnabled(True)
            self.lineEditA2.setEnabled(False)
            self.lineEditA3.setEnabled(False)
            self.lineEditA4.setEnabled(False)
            self.lineEditA5.setEnabled(False)
            self.lineEditA6.setEnabled(False)
            self.lineEditA7.setEnabled(False)
            self.lineEditA8.setEnabled(False)

        elif nclus == 2:
            self.comboBoxC1.setEnabled(True)
            self.comboBoxC2.setEnabled(True)
            self.comboBoxC3.setEnabled(False)
            self.comboBoxC4.setEnabled(False)
            self.comboBoxC5.setEnabled(False)
            self.comboBoxC6.setEnabled(False)
            self.comboBoxC7.setEnabled(False)
            self.comboBoxC8.setEnabled(False)

            self.lineEditA1.setEnabled(True)
            self.lineEditA2.setEnabled(True)
            self.lineEditA3.setEnabled(False)
            self.lineEditA4.setEnabled(False)
            self.lineEditA5.setEnabled(False)
            self.lineEditA6.setEnabled(False)
            self.lineEditA7.setEnabled(False)
            self.lineEditA8.setEnabled(False)

        elif nclus == 3:
            self.comboBoxC1.setEnabled(True)
            self.comboBoxC2.setEnabled(True)
            self.comboBoxC3.setEnabled(True)
            self.comboBoxC4.setEnabled(False)
            self.comboBoxC5.setEnabled(False)
            self.comboBoxC6.setEnabled(False)
            self.comboBoxC7.setEnabled(False)
            self.comboBoxC8.setEnabled(False)

            self.lineEditA1.setEnabled(True)
            self.lineEditA2.setEnabled(True)
            self.lineEditA3.setEnabled(True)
            self.lineEditA4.setEnabled(False)
            self.lineEditA5.setEnabled(False)
            self.lineEditA6.setEnabled(False)
            self.lineEditA7.setEnabled(False)
            self.lineEditA8.setEnabled(False)

        elif nclus == 4:
            self.comboBoxC1.setEnabled(True)
            self.comboBoxC2.setEnabled(True)
            self.comboBoxC3.setEnabled(True)
            self.comboBoxC4.setEnabled(True)
            self.comboBoxC5.setEnabled(False)
            self.comboBoxC6.setEnabled(False)
            self.comboBoxC7.setEnabled(False)
            self.comboBoxC8.setEnabled(False)

            self.lineEditA1.setEnabled(True)
            self.lineEditA2.setEnabled(True)
            self.lineEditA3.setEnabled(True)
            self.lineEditA4.setEnabled(True)
            self.lineEditA5.setEnabled(False)
            self.lineEditA6.setEnabled(False)
            self.lineEditA7.setEnabled(False)
            self.lineEditA8.setEnabled(False)

        elif nclus == 5:
            self.comboBoxC1.setEnabled(True)
            self.comboBoxC2.setEnabled(True)
            self.comboBoxC3.setEnabled(True)
            self.comboBoxC4.setEnabled(True)
            self.comboBoxC5.setEnabled(True)
            self.comboBoxC6.setEnabled(False)
            self.comboBoxC7.setEnabled(False)
            self.comboBoxC8.setEnabled(False)

            self.lineEditA1.setEnabled(True)
            self.lineEditA2.setEnabled(True)
            self.lineEditA3.setEnabled(True)
            self.lineEditA4.setEnabled(True)
            self.lineEditA5.setEnabled(True)
            self.lineEditA6.setEnabled(False)
            self.lineEditA7.setEnabled(False)
            self.lineEditA8.setEnabled(False)

        elif nclus == 6:
            self.comboBoxC1.setEnabled(True)
            self.comboBoxC2.setEnabled(True)
            self.comboBoxC3.setEnabled(True)
            self.comboBoxC4.setEnabled(True)
            self.comboBoxC5.setEnabled(True)
            self.comboBoxC6.setEnabled(True)
            self.comboBoxC7.setEnabled(False)
            self.comboBoxC8.setEnabled(False)

            self.lineEditA1.setEnabled(True)
            self.lineEditA2.setEnabled(True)
            self.lineEditA3.setEnabled(True)
            self.lineEditA4.setEnabled(True)
            self.lineEditA5.setEnabled(True)
            self.lineEditA6.setEnabled(True)
            self.lineEditA7.setEnabled(False)
            self.lineEditA8.setEnabled(False)


        elif nclus == 7:
            self.comboBoxC1.setEnabled(True)
            self.comboBoxC2.setEnabled(True)
            self.comboBoxC3.setEnabled(True)
            self.comboBoxC4.setEnabled(True)
            self.comboBoxC5.setEnabled(True)
            self.comboBoxC6.setEnabled(True)
            self.comboBoxC7.setEnabled(True)
            self.comboBoxC8.setEnabled(False)

            self.lineEditA1.setEnabled(True)
            self.lineEditA2.setEnabled(True)
            self.lineEditA3.setEnabled(True)
            self.lineEditA4.setEnabled(True)
            self.lineEditA5.setEnabled(True)
            self.lineEditA6.setEnabled(True)
            self.lineEditA7.setEnabled(True)
            self.lineEditA8.setEnabled(False)


        elif nclus == 8:
            self.comboBoxC1.setEnabled(True)
            self.comboBoxC2.setEnabled(True)
            self.comboBoxC3.setEnabled(True)
            self.comboBoxC4.setEnabled(True)
            self.comboBoxC5.setEnabled(True)
            self.comboBoxC6.setEnabled(True)
            self.comboBoxC7.setEnabled(True)
            self.comboBoxC8.setEnabled(True)

            self.lineEditA1.setEnabled(True)
            self.lineEditA2.setEnabled(True)
            self.lineEditA3.setEnabled(True)
            self.lineEditA4.setEnabled(True)
            self.lineEditA5.setEnabled(True)
            self.lineEditA6.setEnabled(True)
            self.lineEditA7.setEnabled(True)
            self.lineEditA8.setEnabled(True)


        elif nclus > 8:
            self.comboBoxC1.setEnabled(False)
            self.comboBoxC2.setEnabled(False)
            self.comboBoxC3.setEnabled(False)
            self.comboBoxC4.setEnabled(False)
            self.comboBoxC5.setEnabled(False)
            self.comboBoxC6.setEnabled(False)
            self.comboBoxC7.setEnabled(False)
            self.comboBoxC8.setEnabled(False)

            self.lineEditA1.setEnabled(False)
            self.lineEditA2.setEnabled(False)
            self.lineEditA3.setEnabled(False)
            self.lineEditA4.setEnabled(False)
            self.lineEditA5.setEnabled(False)
            self.lineEditA6.setEnabled(False)
            self.lineEditA7.setEnabled(False)
            self.lineEditA8.setEnabled(False)

        if nclus > 8:
            self.comboBoxColorBig.setEnabled(True)
            self.lineEditAnotClus.setEnabled(False)
        else:
            self.comboBoxColorBig.setEnabled(False)
            self.lineEditAnotClus.setEnabled(True)

    def SaveAverage(self):
        suggested = self.lineEditFilename.text()+'_clus.xls'
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        filesave, _ = QFileDialog.getSaveFileName(self,"QFileDialog.getSaveFileName()",suggested,"Excel Files(*.xls)", options=options)
        if filesave:
            __ , ext = os.path.splitext(filesave)
            if  ext == '.xls':
                filesave = filesave
            else:
                filesave = filesave+'.xls'
            df_save = pd.DataFrame({'Wavenumber':self.wavenumber})
            df_spec = pd.DataFrame(self.spmean.T, columns = [self.label])
            df_save = pd.concat([df_save, df_spec], axis=1, sort=False)
            df_save.to_excel(filesave,index=False)

    def TableAve(self):
        self.Tableview.close()
        self.DataUpdated.emit(np.column_stack((self.wavenumber, self.spmean.T)),0,self.color)
        self.Tableview.show()


    def clear_all(self):
        self.lineEditDirSpectra.setText('')
        self.lineEditDirPurest.setText('')
        self.lineEditFilename.setText('')
        self.lineEditLength.setText('')
        self.comboBoxMethod.setCurrentIndex(0)
        self.comboBoxCmaps.setCurrentIndex(0)
        self.horizontalSliderWavenumber.setSliderPosition(0)
        self.lineEditWavenumber.setText('')
        self.Refresh()
        self.comboBoxMethodCluster.setCurrentIndex(0)
        self.spinBoxNcluster.setValue(8)
        self.lineEditAnotClus.setText('')

        self.comboBoxVisualize.setCurrentIndex(0)

        self.comboBoxC1.setEnabled(False)
        self.comboBoxC2.setEnabled(False)
        self.comboBoxC3.setEnabled(False)
        self.comboBoxC4.setEnabled(False)
        self.comboBoxC5.setEnabled(False)
        self.comboBoxC6.setEnabled(False)
        self.comboBoxC7.setEnabled(False)
        self.comboBoxC8.setEnabled(False)

        self.lineEditA1.setEnabled(False)
        self.lineEditA2.setEnabled(False)
        self.lineEditA3.setEnabled(False)
        self.lineEditA4.setEnabled(False)
        self.lineEditA5.setEnabled(False)
        self.lineEditA6.setEnabled(False)
        self.lineEditA7.setEnabled(False)
        self.lineEditA8.setEnabled(False)

        self.plotMultiVisual.canvas.ax.clear()
        self.plotMultiVisual.canvas.draw()
        self.plot_visual.canvas.ax.clear()
        self.plot_visual.canvas.draw()
        self.plot_specta.canvas.ax.clear()
        self.plot_specta.canvas.draw()
        self.plot_White.canvas.ax.clear()
        self.plot_White.canvas.draw()
        self.plotAverage.canvas.ax.clear()
        self.plotAverage.canvas.draw()
        self.plotCluster.canvas.ax.clear()
        self.plotCluster.canvas.draw()

        self.comboBoxVisualize.clear()
        self.comboBoxVisualize.addItem('Spectra and White Light Image')


    def Refresh(self):
        self.comboBoxC1.setCurrentIndex(0)
        self.comboBoxC2.setCurrentIndex(1)
        self.comboBoxC3.setCurrentIndex(2)
        self.comboBoxC4.setCurrentIndex(3)
        self.comboBoxC5.setCurrentIndex(4)
        self.comboBoxC6.setCurrentIndex(5)
        self.comboBoxC7.setCurrentIndex(6)
        self.comboBoxC8.setCurrentIndex(7)

        self.lineEditA1.setText("")
        self.lineEditA2.setText("")
        self.lineEditA3.setText("")
        self.lineEditA4.setText("")
        self.lineEditA5.setText("")
        self.lineEditA6.setText("")
        self.lineEditA7.setText("")
        self.lineEditA8.setText("")



    def lock_un(self,stat):
        self.pushButtonLoadConc.setEnabled(stat)
        self.pushButtonPrevious.setEnabled(stat)
        self.pushButtonNext.setEnabled(stat)
        self.pushButtonExpandSpectra.setEnabled(stat)
        self.comboBoxMethod.setEnabled(stat)
        self.comboBoxCmaps.setEnabled(stat)
        self.pushButtonExpandProjection.setEnabled(stat)
        self.comboBoxVisualize.setEnabled(stat)
        self.pushButtonExpandVisual.setEnabled(stat)
        self.pushButtonExpandAve.setEnabled(stat)
        self.pushButtonSaveSpectra.setEnabled(stat)
        self.pushButtonExpandCluster.setEnabled(stat)
        self.pushButtonRefresh.setEnabled(stat)
        self.comboBoxMethodCluster.setEnabled(stat)
        self.spinBoxNcluster.setEnabled(stat)
        self.lineEditAnotClus.setEnabled(stat)
        self.pushButtonCluster.setEnabled(stat)
        self.pushButtonReduce.setEnabled(stat)
        self.comboBoxNormalization.setEnabled(stat)
        self.pushButtonSC.setEnabled(stat)
        self.pushButtonTable.setEnabled(stat)



def main():
    parser = argparse.ArgumentParser(
            description='Graphical application for clustering of MCR-ALS output.')
    MyMainWindow.run_octavvs_application(parser=parser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Nov 21 11:41:29 2019

@author: EelkeSpaak
"""

import io
import matplotlib.pyplot as plt
from PyQt5.QtWidgets import QApplication
from PyQt5.QtGui import QImage


def add_clipboard_to_figures():
    oldfig = plt.figure
    def newfig(*args, **kwargs):
        fig = oldfig(*args, **kwargs)
        def clipboard_handler(event):
            if event.key == 'ctrl+c':
                buf = io.BytesIO()
                fig.savefig(buf)
                QApplication.clipboard().setImage(QImage.fromData(buf.getvalue()))
                buf.close()
        fig.canvas.mpl_connect('key_press_event', clipboard_handler)
        return fig
    plt.figure = newfig




"""
Metrics used in pyMCR

All functions must take C, ST, D_actual, D_calculated
"""

def mse(C, ST, D_actual, D_calculated):
    """ Mean square error"""
    return ((D_actual - D_calculated)**2).sum()/D_actual.size



from .spectraldata import SpectralData
from .decompositiondata import DecompositionData
from .opusreader import OpusReader
from .ptirreader import PtirReader
from .image import Image
from .parameters import Parameters


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb 11 16:00:35 2020

@author: carl
"""

from io import BytesIO
from PyQt5.QtWidgets import QSizePolicy
#from PyQt5.QtCore import pyqtSignal, pyqtSlot
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt
# import matplotlib.patches as patches


class WhiteLightWidget(FigureCanvas):
    def __init__(self, parent=None):
        self.fig = Figure()
        self.ax = self.fig.add_subplot(111)
        self.ax.set_axis_off()
        self.ax.set_position([0,0,1,1])
        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        # FigureCanvas.setSizePolicy(self, QSizePolicy.Expanding, QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)
        self.img = None # Image data
        self.image = None # Image object
        # self.points = []

    # def draw_rectangles(self, pixelxy):
    #     for p in self.points:
    #         p.remove()
    #     self.points = []
    #     if pixelxy is None or self.image is None:
    #         return
    #     ixy = self.image.xy
    #     iwh = self.image.wh
    #     if not ixy or not iwh:
    #         return
    #     scx = self.ax.get_xlim()[1]
    #     scy = self.ax.get_ylim()[0]
    #     i = 0
    #     for xy in pixelxy:
    #         x = (xy[0] - ixy[0]) / iwh[0] + .5
    #         y = (xy[1] - ixy[1]) / iwh[1] + .5
    #         if x < 0 or x > 1 or y < 0 or y > 1:
    #             continue
    #         p = patches.Rectangle((x*scx, y*scy), 5, 5, fill=False,
    #                               color='b')
    #         self.ax.add_patch(p)
    #         self.points.append(p)
    #         i = i + 1
    #     self.draw_idle()

    def load(self, filename=None, image=None):
        self.ax.clear()
        self.ax.set_axis_off()
        self.img = None
        self.image = image

        if image is not None:
            if image.fmt is not None:
                self.img = plt.imread(BytesIO(image.data), format=image.fmt)
            elif image.data is not None:
                self.img = image.data
        elif filename is not None:
            try:
                self.img = plt.imread(filename)
            except FileNotFoundError:
                pass
        if self.img is not None:
            self.ax.imshow(self.img, cmap='terrain')
        self.draw_idle()

    def is_loaded(self):
        return self.img is not None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jan  9 16:16:44 2020

@author: carl
"""

from struct import unpack
import numpy as np
from .image import Image

class OpusReader:
    """
    A simple class to extract data from an OPUS file. This is meant as an alternative to
        .mat export and will only load the data matrix and wavenumbers plus the white light image.

    Member variables:
        wh: image dimensions (w, h)
        wavenum: array of wavenumbers
        AB: data matrix, array(w*h, len(wn))
        image: tuple (img_data_bytes, img_type_str)
    """

    def __init__(self, filename=None):
        """
        filename: OPUS file to load data from
        """
        self.wh = (0, 0)
        self.wavenum = None
        self.AB = None
        self.images = []
        if filename is not None:
            self.load(filename)

    def read_chunk(self, file, chunksize, chunkoffset):
        file.seek(chunkoffset)
        return file.read(4 * chunksize)

    def parse_params(self, chunk):
        cix = 0
        inttypes = {0: 'B', 1: '<H', 2: '<I'}
        params = {}
        while cix < len(chunk):
            pname = chunk[cix:cix+3].decode()
            ptype = unpack('<H', chunk[cix+4:cix+6])[0]
            psize = unpack('<H', chunk[cix+6:cix+8])[0]
            pend = cix+8+2*psize
            pdata = chunk[cix+8:pend]
            cix = pend
            if pname == 'END':
                return params, cix
            if ptype == 0:
                params[pname] = unpack(inttypes[psize], pdata)[0]
            elif ptype == 1:
                params[pname] = unpack('<f' if psize == 2 else '<d', pdata)[0]
#            print('pname',pname, ptype, psize)
        return params, cix

    def parse_image(self, chunk):
        p = self.parse_params(chunk[12:])[0]
        jump = p['F00']
        p, q = self.parse_params(chunk[jump:])
        imgtype = 'jpg' if p['V08'] == 258 else 'bmp'
        q = q + jump + (14 if imgtype == 'jpg' else 0)
#        print('pq',p,q)
#        print(chunk[q-16:q+128])
        imgdata = chunk[q:q+p['SRC']]
        return (imgdata, imgtype)

    def load(self, filename):
        f = open(filename, 'rb')
        self.images = []
        HLEN = 504
        header = f.read(HLEN)
        abparams = abmatrix = None
        for ix in range(24, HLEN, 12):
            dt = header[ix]
            db = header[ix+1]
            dtt = header[ix+2]
            dcsize = unpack('<I', header[ix+4:ix+8])[0]
            dcoffs = unpack('<I', header[ix+8:ix+12])[0]
#            print(dt, db, dtt, dcsize, dcoffs)
            if not dcoffs:
                break
            if dt == 31:
                abparams = self.parse_params(self.read_chunk(f, dcsize, dcoffs))[0]
            elif dt == 15:
                abmatrix = self.read_chunk(f, dcsize, dcoffs)
            elif dt == 0 and db == 0 and dtt == 160:
                im = self.parse_image(self.read_chunk(f, dcsize, dcoffs))
                self.images.append(Image(data=im[0], fmt=im[1]))
        # Assume the data were read ok
        start, pad = 16006, 38
        wns, w, h, wn1, wn2 = [abparams[k] for k in ['NPT', 'NPX', 'NPY', 'FXV', 'LXV']]
        n = w * h * (wns + pad)
        raw = np.frombuffer(abmatrix, dtype=np.float32)
        self.AB = raw[start:start+n].reshape((w * h, wns + pad))[:, :wns]
        self.wavenum = np.linspace(wn1, wn2, wns)
        self.wh = (w, h)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  4 16:24:50 2020

@author: carl
"""

#import os.path
from pkg_resources import resource_filename
import numpy as np
from scipy.interpolate import PchipInterpolator
#from scipy.io import loadmat
from pymatreader import read_mat


def load_reference(wn, what=None, matfilename=None):
    """
    Loads and normalizes a spectrum from a Matlab file, interpolating at the given points.
        The reference is assumed to cover the entire range of wavenumbers.
    Parameters:
        wn: array of wavenumbers at which to get the spectrum
        what: A string defining what type of reference to get, corresponding to a file in the
        'reference' directory
        matfilename: the name of an arbitrary Matlab file to load data from; the data must be
        in a matrix called AB, with wavenumbers in the first column.
        Returns: spectrum at the points given by wn
    """
    if (what is None) == (matfilename is None):
        raise ValueError("Either 'what' or 'matfilename' must be specified")
    if what is not None:
        matfilename = resource_filename('octavvs.reference_spectra', what + ".mat")
#        matfilename = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__),
#                                       'reference', what + '.mat'))
    ref = read_mat(matfilename)['AB']
    # Handle the case of high-to-low since the interpolator requires low-to-high
    d = 1 if ref[0,0] < ref[-1,0] else -1
    ref = PchipInterpolator(ref[::d,0], ref[::d,1])(wn)
    return ref #/ ref.max()

def nonnegative(y, fracspectra=0, fracvalues=0):
    """
    Make a matrix of spectral data nonnegative by shifting all the spectra up by the same computed
    amount, followed by setting negative values to 0. The shift is chosen such that at most
    fracspectra of the spectra get more than fracvalues of their intensities set to zero.
    Parameters:
    y: array of intensities for (pixel, wavenumber)
    fracspectra: unheeded fraction of the spectra
    fracvalues: maximal fraction of points to clip at 0
    Returns: shifted spectra in the same format as y
    """
    s = int(fracspectra * y.shape[0])
    v = int(fracvalues * y.shape[1])
    if s == 0 or v == 0:
        return y - np.min(y.min(), 0)
    if s >= y.shape[0] or v >= y.shape[1]:
        return np.maximum(y, 0)
    yp = np.partition(y, v, axis=1)[:,v]
    a = np.partition(yp, s)[s]
    return np.maximum(y - a if a < 0 else y, 0)

def pca_nipals(x, ncomp, tol=1e-5, max_iter=1000, copy=True, explainedvariance=None):
    """
    NIPALS algorithm for PCA, based on the code in statmodels.multivariate
    but with optimizations as in Bassan's Matlab implementation to
    sacrifice some accuracy for speed.
    x: ndarray of data, will be altered
    ncomp: number of PCA components to return
    tol: tolerance
    copy: If false, destroy the input matrix x
    explainedvariance: If >0, stop after this fraction of the total variance is explained
    returns: PCA loadings as rows
    """
    if copy:
        x = x.copy()
    if explainedvariance is not None and explainedvariance > 0:
        varlim = (x * x).sum() * (1. - explainedvariance)
    else:
        varlim = 0
    npts, nvar = x.shape
    vecs = np.empty((ncomp, nvar))
    for i in range(ncomp):
        factor = np.ones(npts)
        for j in range(max_iter):
            vec = x.T @ factor #/ (factor @ factor)
            vec = vec / np.sqrt(vec @ vec)
            f_old = factor
            factor = x @ vec #/ (vec @ vec)
            f_old = factor - f_old
            if tol > np.sqrt(f_old @ f_old) / np.sqrt(factor @ factor):
                break
        vecs[i, :] = vec
        if i < ncomp - 1:
            x -= factor[:,None] @ vec[None,:]
            if varlim > 0 and (x * x).sum() < varlim:
                return vecs[:i+1, :]
    return vecs

def find_wn_ranges(wn, ranges):
    """
    Find indexes corresponding to the beginning and end of a list of ranges of wavenumbers. The
    wavenumbers have to be sorted in either direction.
    Parameters:
    wn: array of wavenumbers
    ranges: numpy array of shape (n, 2) with desired wavenumber ranges in order [low,high]
    Returns: numpy array of shape (n, 2) with indexes of the wavenumbers delimiting those ranges
    """
    if isinstance(ranges, list):
        ranges = np.array(ranges)
    if(wn[0] < wn[-1]):
        return np.stack((np.searchsorted(wn, ranges[:,0]),
                         np.searchsorted(wn, ranges[:,1], 'right')), 1)
    return len(wn) - np.stack((np.searchsorted(wn[::-1], ranges[:,1], 'right'),
                              np.searchsorted(wn[::-1], ranges[:,0])), 1)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec  4 14:45:19 2019

@author: carl
"""

import sys
import traceback
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import Qt

class ExceptionDialog:
    def install(qwin):
        "installs a graphical exception handler that shows a message and doesn't terminate"
        def ehook(exctype, value, tb):
            err = 'Unhandled exception: ' + repr(value)
            details = ''.join(traceback.format_exception(exctype, value, tb))
            print(err, '\n', details)
    
            q = QMessageBox(qwin)
            q.setIcon(QMessageBox.Critical)
            q.setWindowTitle("Error")
            q.setText(err)
            q.setTextFormat(Qt.PlainText)
            q.setDetailedText(str(details))
            q.addButton('OK', QMessageBox.AcceptRole)
            return q.exec()
    
        oldhook = sys.excepthook
        sys.excepthook = ehook
        return oldhook

